@@ -23,7 +23,7 @@ from ramses_rf.protocol.protocol import MessageProtocol, MessageTransport
 from ramses_rf.protocol.transport import PacketProtocolBase
 
 from .const import DONT_CREATE_MESSAGES, SZ_DEVICE_ID, SZ_DEVICES, __dev_mode__
-from .device import Device, zx_device_factory
+from .device import Device, device_factory
 from .helpers import schedule_task, shrink
 from .processor import Message, process_msg
 from .protocol import (
@@ -37,8 +37,9 @@ from .protocol import (
     set_pkt_logging_config,
 )
 from .protocol.address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR
+from .protocol.schemas import SZ_PACKET_LOG, SZ_PORT_CONFIG, SZ_PORT_NAME
 from .schemas import (
-    SCH_GLOBAL_GATEWAY_CONFIG,
+    SCH_GLOBAL_CONFIG,
     SCH_TRAITS,
     SZ_ALIAS,
     SZ_BLOCK_LIST,
@@ -47,7 +48,7 @@ from .schemas import (
     SZ_ENFORCE_KNOWN_LIST,
     SZ_FAKED,
     SZ_KNOWN_LIST,
-    SZ_MAIN_CONTROLLER,
+    SZ_MAIN_TCS,
     SZ_ORPHANS,
     load_config,
     load_schema,
@@ -83,13 +84,13 @@ class Engine:
 
     def __init__(
         self,
-        serial_port: None | str,
+        port_name: None | str,
         input_file: None | TextIO = None,
         port_config: None | dict = None,
         loop: None | asyncio.AbstractEventLoop = None,
     ) -> None:
 
-        self.ser_name = serial_port
+        self.ser_name = port_name
         self._input_file = input_file
         self._port_config = port_config or {}
         self._loop = loop or asyncio.get_running_loop()
@@ -152,9 +153,9 @@ class Engine:
         )
 
         if self.ser_name:
-            source = {"ser_port": self.ser_name}
+            source = {SZ_PORT_NAME: self.ser_name, SZ_PORT_CONFIG: self._port_config}
         else:
-            source = {"packet_log": self._input_file}
+            source = {SZ_PACKET_LOG: self._input_file}
 
         self.pkt_protocol, self.pkt_transport = self._create_pkt_stack(
             pkt_receiver, **source
@@ -290,6 +291,11 @@ class Engine:
             except TimeoutError as exc:  # raised by send_cmd()
                 raise TimeoutError(f"cmd ({cmd.tx_header}) timed out: {exc}")
 
+            # except RuntimeError as exc:  # raised by send_cmd()
+            #     _LOGGER.error(f"cmd ({cmd.tx_header}) raised an exception: {exc!r}")
+            #     if self.msg_transport.is_closing:
+            #         pass
+
             except Exception as exc:
                 _LOGGER.error(f"cmd ({cmd.tx_header}) raised an exception: {exc!r}")
                 raise exc
@@ -306,20 +312,20 @@ class Gateway(Engine):
 
     def __init__(
         self,
-        serial_port: None | str,
+        port_name: None | str,
         debug_mode: None | bool = None,
         input_file: None | TextIO = None,
-        port_config: None | dict = None,
         loop: None | asyncio.AbstractEventLoop = None,
+        port_config: None | dict = None,
         **kwargs,
     ) -> None:
 
         if debug_mode:
             _LOGGER.setLevel(logging.DEBUG)  # should be INFO?
-        _LOGGER.debug("Starting RAMSES RF, **kwargs = %s", kwargs)
+        _LOGGER.debug("Starting RAMSES RF, **config = %s", kwargs)
 
         super().__init__(
-            serial_port, input_file=input_file, port_config=port_config, loop=loop
+            port_name, input_file=input_file, port_config=port_config, loop=loop
         )
 
         self._tasks: list = []  # TODO: used by discovery, move lower?
@@ -328,9 +334,7 @@ class Gateway(Engine):
         (self.config, self._schema, self._include, self._exclude) = load_config(
             self.ser_name,
             self._input_file,
-            **SCH_GLOBAL_GATEWAY_CONFIG(
-                {k: v for k, v in kwargs.items() if k[:1] != "_"}
-            ),
+            **SCH_GLOBAL_CONFIG({k: v for k, v in kwargs.items() if k[:1] != "_"}),
         )
         set_pkt_logging_config(
             cc_console=self.config.reduce_processing >= DONT_CREATE_MESSAGES,
@@ -534,7 +538,7 @@ class Gateway(Engine):
 
             if dev_id in self._unwanted:
                 # _LOGGER.warning()
-                raise LookupError(f"Can't create {dev_id}: it unwanted or invalid")
+                raise LookupError(f"Can't create {dev_id}: it is unwanted or invalid")
 
             if self.config.enforce_known_list and (
                 dev_id not in self._include
@@ -558,7 +562,7 @@ class Gateway(Engine):
 
         dev = self.device_by_id.get(dev_id)
         if not dev:
-            dev = zx_device_factory(self, Address(dev_id), msg=msg, **traits)
+            dev = device_factory(self, Address(dev_id), msg=msg, **traits)
 
         # TODO: the exact order of the follwoing may need refining...
 
@@ -638,7 +642,7 @@ class Gateway(Engine):
 
         return {
             "_gateway_id": self.hgi.id if self.hgi else None,
-            SZ_MAIN_CONTROLLER: self.tcs.id if self.tcs else None,
+            SZ_MAIN_TCS: self.tcs.id if self.tcs else None,
             SZ_CONFIG: {SZ_ENFORCE_KNOWN_LIST: self.config.enforce_known_list},
             SZ_KNOWN_LIST: self.known_list,
             SZ_BLOCK_LIST: [{k: v} for k, v in self._exclude.items()],
@@ -655,7 +659,7 @@ class Gateway(Engine):
         just like the other devices in the schema.
         """
 
-        schema = {SZ_MAIN_CONTROLLER: self.tcs.ctl.id if self.tcs else None}
+        schema = {SZ_MAIN_TCS: self.tcs.ctl.id if self.tcs else None}
 
         for tcs in self.systems:
             schema[tcs.ctl.id] = tcs.schema
@@ -733,26 +737,3 @@ class Gateway(Engine):
         self._tasks.append(
             schedule_task(fnc, *args, delay=delay, period=period, **kwargs)
         )
-
-
-class GatewayWithSql(Gateway):
-    """The gateway class (with MySQL)."""
-
-    def __init__(self, *args, **kwargs) -> None:
-
-        import sqlite3
-
-        self._db: sqlite3.Connection = sqlite3.connect("file::memory:?cache=shared")
-        self._cur = self._db.cursor()
-
-    async def start(self, *, start_discovery=True) -> None:
-        await super().start(start_discovery=start_discovery)
-
-    async def stop(self) -> None:
-        await super().stop()
-
-    def _get_state(self, include_expired=None) -> tuple[dict, dict]:
-        pass
-
-    async def _set_state(self, packets, *, schema=None) -> None:
-        pass
