@@ -40,8 +40,17 @@ from .const import (
     __dev_mode__,
 )
 from .exceptions import ExpiredCallbackError
-from .frame import Frame, _CodeT, _DevIdT, _HeaderT, _PayloadT, _VerbT, pkt_header
-from .helpers import dt_now, dtm_to_hex, str_to_hex, temp_to_hex, timestamp, typechecked
+from .frame import Frame, _CodeT, _DeviceIdT, _HeaderT, _PayloadT, _VerbT, pkt_header
+from .helpers import (
+    bool_from_hex,
+    double_to_hex,
+    dt_now,
+    dtm_to_hex,
+    str_to_hex,
+    temp_to_hex,
+    timestamp,
+    typechecked,
+)
 from .opentherm import parity
 from .parsers import LOOKUP_PUZZ
 from .ramses import _2411_PARAMS_SCHEMA
@@ -277,7 +286,7 @@ class Command(Frame):
     They have QoS and/or callbacks (but no RSSI).
     """
 
-    def __init__(self, frame: str, qos: dict = None, cbk: dict = None) -> None:
+    def __init__(self, frame: str, qos: dict = None, callback: dict = None) -> None:
         """Create a command from a string (and its meta-attrs).
 
         Will raise InvalidPacketError if it is invalid.
@@ -286,7 +295,7 @@ class Command(Frame):
         super().__init__(frame)  # may raise InvalidPacketError if it is invalid
 
         # used by app layer: callback (protocol.py: func, args, daemon, timeout)
-        self._cbk = cbk or {}
+        self._cbk = callback or {}
         # used by pkt layer: qos (transport.py: backoff, priority, retries, timeout)
         self._qos = _qos_params(self.verb, self.code, qos or {})
 
@@ -463,10 +472,10 @@ class Command(Frame):
     @validate_api_params()
     def set_bypass_position(
         cls,
-        fan_id: _DevIdT,
+        fan_id: _DeviceIdT,
         *,
         bypass_position: float = None,
-        src_id: _DevIdT = None,
+        src_id: _DeviceIdT = None,
         **kwargs,
     ):
         """Constructor to set the position of the bypass valve (c.f. parser_22f7).
@@ -505,11 +514,11 @@ class Command(Frame):
     @validate_api_params()
     def set_fan_param(
         cls,
-        fan_id: _DevIdT,
+        fan_id: _DeviceIdT,
         param_id: str,
         value: str,
         *,
-        src_id: _DevIdT = None,
+        src_id: _DeviceIdT = None,
         **kwargs,
     ):
         """Constructor to set a configurable fan parameter (c.f. parser_2411)."""
@@ -530,11 +539,11 @@ class Command(Frame):
     @validate_api_params()
     def set_fan_mode(
         cls,
-        fan_id: _DevIdT,
+        fan_id: _DeviceIdT,
         fan_mode,
         *,
         seqn: int = None,
-        src_id: _DevIdT = None,
+        src_id: _DeviceIdT = None,
         idx: str = "00",  # could be e.g. "63"
         **kwargs,
     ):
@@ -549,7 +558,7 @@ class Command(Frame):
         #  - are cast as a triplet, 0.1s apart?, with a seqn (000-255) and no src_id
         #  - triplet has same seqn, increased monotonically mod 256 after every triplet
         #  - only payloads seen: '(00|63)0[234]04', may accept '000.'
-        #  I 218 --:------ --:------ 39:159057 22F1 003 000204  # low
+        # .I 218 --:------ --:------ 39:159057 22F1 003 000204  # low
 
         # Scheme 1a: I --- --:------ --:------ 21:038634 (less common)
         #  - some systems that accept scheme 2 will accept this scheme
@@ -557,7 +566,7 @@ class Command(Frame):
         # Scheme 2: I --- 21:038634 18:126620 --:------ (less common)
         #  - are cast as a triplet, 0.085s apart, without a seqn (i.e. is ---)
         #  - only payloads seen: '000.0[47A]', may accept '000.'
-        #  I --- 21:038634 18:126620 --:------ 22F1 003 000507
+        # .I --- 21:038634 18:126620 --:------ 22F1 003 000507
 
         from .ramses import _22F1_MODE_ORCON
 
@@ -593,7 +602,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|1F41
     @typechecked
     @validate_api_params()
-    def get_dhw_mode(cls, ctl_id: _DevIdT, **kwargs):
+    def get_dhw_mode(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the mode of the DHW (c.f. parser_1f41)."""
 
         dhw_idx = f"{kwargs.pop(SZ_DHW_IDX, 0):02X}"  # only 00 or 01 (rare)
@@ -604,7 +613,7 @@ class Command(Frame):
     @validate_api_params()
     def set_dhw_mode(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         *,
         mode: Union[None, int, str] = None,
         active: bool = None,
@@ -643,7 +652,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|10A0
     @typechecked
     @validate_api_params()
-    def get_dhw_params(cls, ctl_id: _DevIdT, **kwargs):
+    def get_dhw_params(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the params of the DHW (c.f. parser_10a0)."""
 
         dhw_idx = f"{kwargs.pop(SZ_DHW_IDX, 0):02X}"  # only 00 or 01 (rare)
@@ -654,7 +663,7 @@ class Command(Frame):
     @typechecked
     def set_dhw_params(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         *,
         setpoint: float = 50.0,
         overrun: int = 5,
@@ -692,7 +701,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|1260
     @typechecked
     @validate_api_params()
-    def get_dhw_temp(cls, ctl_id: _DevIdT, **kwargs):
+    def get_dhw_temp(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the temperature of the DHW sensor (c.f. parser_10a0)."""
 
         dhw_idx = f"{kwargs.pop(SZ_DHW_IDX, 0):02X}"  # only 00 or 01 (rare)
@@ -701,7 +710,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|1030
     @typechecked
     @validate_api_params(has_zone=True)
-    def get_mix_valve_params(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, **kwargs):
+    def get_mix_valve_params(cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, **kwargs):
         """Constructor to get the mix valve params of a zone (c.f. parser_1030)."""
 
         return cls.from_attrs(
@@ -713,7 +722,7 @@ class Command(Frame):
     @validate_api_params(has_zone=True)
     def set_mix_valve_params(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         zone_idx: _ZoneIdxT,
         *,
         max_flow_setpoint=55,
@@ -753,7 +762,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|3220
     @typechecked
     @validate_api_params()
-    def get_opentherm_data(cls, otb_id: _DevIdT, msg_id: Union[int, str], **kwargs):
+    def get_opentherm_data(cls, otb_id: _DeviceIdT, msg_id: Union[int, str], **kwargs):
         """Constructor to get (Read-Data) opentherm msg value (c.f. parser_3220)."""
 
         msg_id = msg_id if isinstance(msg_id, int) else int(msg_id, 16)
@@ -763,7 +772,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|0008
     @typechecked
     @validate_api_params()  # has_zone is optional
-    def get_relay_demand(cls, dev_id: _DevIdT, zone_idx: _ZoneIdxT = None, **kwargs):
+    def get_relay_demand(cls, dev_id: _DeviceIdT, zone_idx: _ZoneIdxT = None, **kwargs):
         """Constructor to get the demand of a relay/zone (c.f. parser_0008)."""
 
         payload = "00" if zone_idx is None else f"{zone_idx:02X}"
@@ -772,7 +781,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|0006
     @typechecked
     @validate_api_params()
-    def get_schedule_version(cls, ctl_id: _DevIdT, **kwargs):
+    def get_schedule_version(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the current version (change counter) of the schedules.
 
         This number is increased whenever any zone's schedule is changed (incl. the DHW
@@ -787,7 +796,7 @@ class Command(Frame):
     @validate_api_params(has_zone=True)
     def get_schedule_fragment(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         zone_idx: _ZoneIdxT,
         frag_number: int,
         total_frags: Optional[int],
@@ -826,7 +835,7 @@ class Command(Frame):
     @validate_api_params(has_zone=True)
     def set_schedule_fragment(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         zone_idx: _ZoneIdxT,
         frag_num: int,
         frag_cnt: int,
@@ -853,7 +862,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|0100
     @typechecked
     @validate_api_params()
-    def get_system_language(cls, ctl_id: _DevIdT, **kwargs):
+    def get_system_language(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the language of a system (c.f. parser_0100)."""
 
         return cls.from_attrs(RQ, ctl_id, Code._0100, "00", **kwargs)
@@ -861,7 +870,9 @@ class Command(Frame):
     @classmethod  # constructor for RQ|0418
     @typechecked
     @validate_api_params()
-    def get_system_log_entry(cls, ctl_id: _DevIdT, log_idx: Union[int, str], **kwargs):
+    def get_system_log_entry(
+        cls, ctl_id: _DeviceIdT, log_idx: Union[int, str], **kwargs
+    ):
         """Constructor to get a log entry from a system (c.f. parser_0418)."""
 
         log_idx = log_idx if isinstance(log_idx, int) else int(log_idx, 16)
@@ -870,7 +881,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|2E04
     @typechecked
     @validate_api_params()
-    def get_system_mode(cls, ctl_id: _DevIdT, **kwargs):
+    def get_system_mode(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the mode of a system (c.f. parser_2e04)."""
 
         return cls.from_attrs(RQ, ctl_id, Code._2E04, FF, **kwargs)
@@ -880,7 +891,7 @@ class Command(Frame):
     @validate_api_params()
     def set_system_mode(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         system_mode,
         *,
         until: Union[None, dt, str] = None,
@@ -918,7 +929,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|313F
     @typechecked
     @validate_api_params()
-    def get_system_time(cls, ctl_id: _DevIdT, **kwargs):
+    def get_system_time(cls, ctl_id: _DeviceIdT, **kwargs):
         """Constructor to get the datetime of a system (c.f. parser_313f)."""
 
         return cls.from_attrs(RQ, ctl_id, Code._313F, "00", **kwargs)
@@ -928,13 +939,13 @@ class Command(Frame):
     @validate_api_params()
     def set_system_time(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         datetime: Union[dt, str],
         is_dst: Optional[bool] = False,
         **kwargs,
     ):
         """Constructor to set the datetime of a system (c.f. parser_313f)."""
-        #  W --- 30:185469 01:037519 --:------ 313F 009 0060003A0C1B0107E5
+        # .W --- 30:185469 01:037519 --:------ 313F 009 0060003A0C1B0107E5
 
         dt_str = dtm_to_hex(datetime, is_dst=is_dst, incl_seconds=True)
         return cls.from_attrs(W_, ctl_id, Code._313F, f"0060{dt_str}", **kwargs)
@@ -942,7 +953,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|1100
     @typechecked
     @validate_api_params()
-    def get_tpi_params(cls, dev_id: _DevIdT, *, domain_id=None, **kwargs):
+    def get_tpi_params(cls, dev_id: _DeviceIdT, *, domain_id=None, **kwargs):
         """Constructor to get the TPI params of a system (c.f. parser_1100)."""
 
         if domain_id is None:
@@ -954,7 +965,7 @@ class Command(Frame):
     @validate_api_params()
     def set_tpi_params(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         domain_id: Optional[str],
         *,
         cycle_rate: int = 3,  # TODO: check
@@ -990,7 +1001,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|000A
     @typechecked
     @validate_api_params(has_zone=True)
-    def get_zone_config(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, **kwargs):
+    def get_zone_config(cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, **kwargs):
         """Constructor to get the config of a zone (c.f. parser_000a)."""
 
         return cls.from_attrs(
@@ -1002,7 +1013,7 @@ class Command(Frame):
     @validate_api_params(has_zone=True)
     def set_zone_config(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         zone_idx: _ZoneIdxT,
         *,
         min_temp: float = 5,
@@ -1043,7 +1054,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|2349
     @typechecked
     @validate_api_params(has_zone=True)
-    def get_zone_mode(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, **kwargs):
+    def get_zone_mode(cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, **kwargs):
         """Constructor to get the mode of a zone (c.f. parser_2349)."""
 
         return cls.from_attrs(
@@ -1055,7 +1066,7 @@ class Command(Frame):
     @validate_api_params(has_zone=True)
     def set_zone_mode(
         cls,
-        ctl_id: _DevIdT,
+        ctl_id: _DeviceIdT,
         zone_idx: _ZoneIdxT,
         *,
         mode: str = None,
@@ -1076,8 +1087,8 @@ class Command(Frame):
         - mode == Temporary & until is None (will silently ignore ???)
         - until and duration are mutually exclusive
         """
-        #  W --- 18:013393 01:145038 --:------ 2349 013 0004E201FFFFFF330B1A0607E4
-        #  W --- 22:017139 01:140959 --:------ 2349 007 0801F400FFFFFF
+        # .W --- 18:013393 01:145038 --:------ 2349 013 0004E201FFFFFF330B1A0607E4
+        # .W --- 22:017139 01:140959 --:------ 2349 007 0801F400FFFFFF
 
         mode = _normalise_mode(mode, setpoint, until, duration)
 
@@ -1101,7 +1112,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|0004
     @typechecked
     @validate_api_params(has_zone=True)
-    def get_zone_name(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, **kwargs):
+    def get_zone_name(cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, **kwargs):
         """Constructor to get the name of a zone (c.f. parser_0004)."""
 
         return cls.from_attrs(
@@ -1111,7 +1122,9 @@ class Command(Frame):
     @classmethod  # constructor for W|0004
     @typechecked
     @validate_api_params(has_zone=True)
-    def set_zone_name(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, name: str, **kwargs):
+    def set_zone_name(
+        cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, name: str, **kwargs
+    ):
         """Constructor to set the name of a zone (c.f. parser_0004)."""
 
         payload = f"{zone_idx:02X}00{str_to_hex(name)[:40]:0<40}"
@@ -1121,10 +1134,10 @@ class Command(Frame):
     @typechecked
     @validate_api_params(has_zone=True)
     def set_zone_setpoint(
-        cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, setpoint: float, **kwargs
+        cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, setpoint: float, **kwargs
     ):
         """Constructor to set the setpoint of a zone (c.f. parser_2309)."""
-        #  W --- 34:092243 01:145038 --:------ 2309 003 0107D0
+        # .W --- 34:092243 01:145038 --:------ 2309 003 0107D0
 
         payload = f"{zone_idx:02X}{temp_to_hex(setpoint)}"
         return cls.from_attrs(W_, ctl_id, Code._2309, payload, **kwargs)
@@ -1132,7 +1145,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|30C9
     @typechecked
     @validate_api_params(has_zone=True)
-    def get_zone_temp(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, **kwargs):
+    def get_zone_temp(cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, **kwargs):
         """Constructor to get the current temperature of a zone (c.f. parser_30c9)."""
 
         return cls.from_attrs(RQ, ctl_id, Code._30C9, f"{zone_idx:02X}", **kwargs)
@@ -1140,7 +1153,7 @@ class Command(Frame):
     @classmethod  # constructor for RQ|12B0
     @typechecked
     @validate_api_params(has_zone=True)
-    def get_zone_window_state(cls, ctl_id: _DevIdT, zone_idx: _ZoneIdxT, **kwargs):
+    def get_zone_window_state(cls, ctl_id: _DeviceIdT, zone_idx: _ZoneIdxT, **kwargs):
         """Constructor to get the openwindow state of a zone (c.f. parser_12b0)."""
 
         return cls.from_attrs(RQ, ctl_id, Code._12B0, f"{zone_idx:02X}", **kwargs)
@@ -1150,8 +1163,8 @@ class Command(Frame):
     @validate_api_params()
     def put_actuator_cycle(
         cls,
-        src_id: _DevIdT,
-        dst_id: _DevIdT,
+        src_id: _DeviceIdT,
+        dst_id: _DeviceIdT,
         modulation_level: float,
         actuator_countdown: int,
         *,
@@ -1166,14 +1179,14 @@ class Command(Frame):
 
         if src_id[:2] != DEV_TYPE_MAP.BDR:
             raise TypeError(
-                f"Faked device {src_id} has unsupported device type: "
+                f"Faked device {src_id} has an unsupported device type: "
                 f"device_id should be like {DEV_TYPE_MAP.BDR}:xxxxxx"
             )
 
         payload = "00"
         payload += f"{cycle_countdown:04X}" if cycle_countdown is not None else "7FFF"
         payload += f"{actuator_countdown:04X}"
-        payload += f"{int(modulation_level * 200):02X}FF"
+        payload += f"{int(modulation_level * 200):02X}FF"  # percent_to_hex
         return cls._from_attrs(
             RP, Code._3EF1, payload, addr0=src_id, addr1=dst_id, **kwargs
         )
@@ -1181,17 +1194,17 @@ class Command(Frame):
     @classmethod  # constructor for I|3EF0  # TODO: trap corrupt states?
     @typechecked
     @validate_api_params()
-    def put_actuator_state(cls, dev_id: _DevIdT, modulation_level: float, **kwargs):
+    def put_actuator_state(cls, dev_id: _DeviceIdT, modulation_level: float, **kwargs):
         """Constructor to announce the modulation level of an actuator (3EF0).
 
         This is for use by a faked BDR91A or similar.
         """
-        #  I --- 13:049798 --:------ 13:049798 3EF0 003 00C8FF
-        #  I --- 13:106039 --:------ 13:106039 3EF0 003 0000FF
+        # .I --- 13:049798 --:------ 13:049798 3EF0 003 00C8FF
+        # .I --- 13:106039 --:------ 13:106039 3EF0 003 0000FF
 
         if dev_id[:2] != DEV_TYPE_MAP.BDR:
             raise TypeError(
-                f"Faked device {dev_id} has unsupported device type: "
+                f"Faked device {dev_id} has an unsupported device type: "
                 f"device_id should be like {DEV_TYPE_MAP.BDR}:xxxxxx"
             )
 
@@ -1210,17 +1223,17 @@ class Command(Frame):
         cls,
         verb: _VerbT,
         codes: _CodeT | Iterable[_CodeT],
-        src_id: _DevIdT,
+        src_id: _DeviceIdT,
         *,
         idx="00",
-        dst_id: None | _DevIdT = None,
+        dst_id: None | _DeviceIdT = None,
         **kwargs,
     ):
         """Constructor for RF bind commands (1FC9), for use by faked devices."""
 
-        #  I --- 34:021943 --:------ 34:021943 1FC9 024 00-2309-8855B7 00-1FC9-8855B7
-        #  W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E
-        #  I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
+        # .I --- 34:021943 --:------ 34:021943 1FC9 024 00-2309-8855B7 00-1FC9-8855B7
+        # .W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E
+        # .I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
 
         hex_id = Address.convert_to_hex(src_id)
         codes = ([codes] if isinstance(codes, _CodeT) else list(codes)) + [Code._1FC9]
@@ -1236,7 +1249,7 @@ class Command(Frame):
         else:
             raise ValueError("Invalid parameters")
 
-        kwargs.update({"priority": Priority.HIGH, "retries": 3})
+        kwargs["qos"] = {"priority": Priority.HIGH, "retries": 3}
         return cls._from_attrs(
             verb, Code._1FC9, payload, addr0=src_id, addr1=dst_id, addr2=addr2, **kwargs
         )
@@ -1244,7 +1257,7 @@ class Command(Frame):
     @classmethod  # constructor for I|1260  # TODO: trap corrupt temps?
     @typechecked
     @validate_api_params()
-    def put_dhw_temp(cls, dev_id: _DevIdT, temperature: float, **kwargs):
+    def put_dhw_temp(cls, dev_id: _DeviceIdT, temperature: float, **kwargs):
         """Constructor to announce the current temperature of an DHW sensor (1260).
 
         This is for use by a faked CS92A or similar.
@@ -1254,7 +1267,7 @@ class Command(Frame):
 
         if dev_id[:2] != DEV_TYPE_MAP.DHW:
             raise TypeError(
-                f"Faked device {dev_id} has unsupported device type: "
+                f"Faked device {dev_id} has an unsupported device type: "
                 f"device_id should be like {DEV_TYPE_MAP.DHW}:xxxxxx"
             )
 
@@ -1266,7 +1279,7 @@ class Command(Frame):
     @classmethod  # constructor for I|0002  # TODO: trap corrupt temps?
     @typechecked
     @validate_api_params()
-    def put_outdoor_temp(cls, dev_id: _DevIdT, temperature: float, **kwargs):
+    def put_outdoor_temp(cls, dev_id: _DeviceIdT, temperature: float, **kwargs):
         """Constructor to announce the current temperature of an outdoor sensor (0002).
 
         This is for use by a faked HB85 or similar.
@@ -1274,7 +1287,7 @@ class Command(Frame):
 
         if dev_id[:2] != DEV_TYPE_MAP.OUT:
             raise TypeError(
-                f"Faked device {dev_id} has unsupported device type: "
+                f"Faked device {dev_id} has an unsupported device type: "
                 f"device_id should be like {DEV_TYPE_MAP.OUT}:xxxxxx"
             )
 
@@ -1287,17 +1300,17 @@ class Command(Frame):
     @typechecked
     @validate_api_params()
     def put_sensor_temp(
-        cls, dev_id: _DevIdT, temperature: Union[None, float, int], **kwargs
+        cls, dev_id: _DeviceIdT, temperature: Union[None, float, int], **kwargs
     ):
         """Constructor to announce the current temperature of a thermostat (3C09).
 
         This is for use by a faked DTS92(E) or similar.
         """
-        #  I --- 34:021943 --:------ 34:021943 30C9 003 000C0D
+        # .I --- 34:021943 --:------ 34:021943 30C9 003 000C0D
 
         if dev_id[:2] != DEV_TYPE_MAP.HCW:
             raise TypeError(
-                f"Faked device {dev_id} has unsupported device type: "
+                f"Faked device {dev_id} has an unsupported device type: "
                 f"device_id should be like {DEV_TYPE_MAP.HCW}:xxxxxx"
             )
 
@@ -1306,14 +1319,56 @@ class Command(Frame):
             I_, Code._30C9, payload, addr0=dev_id, addr2=dev_id, **kwargs
         )
 
+    @classmethod  # constructor for I|1298
+    @typechecked
+    @validate_api_params()
+    def put_co2_level(
+        cls, dev_id: _DeviceIdT, co2_level: Union[None, float, int], /, **kwargs
+    ):
+        """Constructor to announce the current co2 level of a sensor (1298)."""
+        # .I --- 37:039266 --:------ 37:039266 1298 003 000316
+
+        payload = f"00{double_to_hex(co2_level)}"
+        return cls._from_attrs(
+            I_, Code._1298, payload, addr0=dev_id, addr2=dev_id, **kwargs
+        )
+
+    @classmethod  # constructor for I|12A0
+    @typechecked
+    @validate_api_params()
+    def put_indoor_humidity(
+        cls, dev_id: _DeviceIdT, indoor_humidity: Union[None, float, int], /, **kwargs
+    ):
+        """Constructor to announce the current humidity of a sensor (12A0)."""
+        # .I --- 37:039266 --:------ 37:039266 1298 003 000316
+
+        payload = f"00{int(indoor_humidity * 100):02X}"  # percent_to_hex
+        return cls._from_attrs(
+            I_, Code._12A0, payload, addr0=dev_id, addr2=dev_id, **kwargs
+        )
+
+    @classmethod  # constructor for I|2E10
+    @typechecked
+    @validate_api_params()
+    def put_presence_detected(
+        cls, dev_id: _DeviceIdT, presence_detected: Union[None, bool], /, **kwargs
+    ):
+        """Constructor to announce the current presence state of a sensor (2E10)."""
+        # .I --- ...
+
+        payload = f"00{bool_from_hex(presence_detected)}"
+        return cls._from_attrs(
+            I_, Code._2E10, payload, addr0=dev_id, addr2=dev_id, **kwargs
+        )
+
     @classmethod  # constructor for internal use only
     @typechecked
-    def _puzzle(cls, msg_type: str = Optional[str], message: str = "", **kwargs):
+    def _puzzle(cls, msg_type: None | str = None, message: str = "", **kwargs):
 
         if msg_type is None:
             msg_type = "12" if message else "10"
 
-        assert msg_type in LOOKUP_PUZZ, "Invalid/deprecated Puzzle type"
+        assert msg_type in LOOKUP_PUZZ, f"Invalid/deprecated Puzzle type: {msg_type}"
 
         qos = kwargs.get("qos", {})
         qos["priority"] = qos.get("priority", Priority.HIGHEST)
@@ -1364,6 +1419,8 @@ CODE_API_MAP = {
     f"{W_}|{Code._1100}": Command.set_tpi_params,
     f"{RQ}|{Code._1260}": Command.get_dhw_temp,
     f"{I_}|{Code._1260}": Command.put_dhw_temp,
+    f"{I_}|{Code._1298}": Command.put_co2_level,
+    f"{I_}|{Code._12A0}": Command.put_indoor_humidity,
     f"{RQ}|{Code._12B0}": Command.get_zone_window_state,
     f"{RQ}|{Code._1F41}": Command.get_dhw_mode,
     f"{W_}|{Code._1F41}": Command.set_dhw_mode,
@@ -1374,6 +1431,7 @@ CODE_API_MAP = {
     f"{W_}|{Code._2411}": Command.set_fan_param,
     f"{RQ}|{Code._2E04}": Command.get_system_mode,
     f"{W_}|{Code._2E04}": Command.set_system_mode,
+    f"{I_}|{Code._2E10}": Command.put_presence_detected,
     f"{I_}|{Code._30C9}": Command.put_sensor_temp,
     f"{RQ}|{Code._30C9}": Command.get_zone_temp,
     f"{RQ}|{Code._313F}": Command.get_system_time,
