@@ -61,6 +61,8 @@ SZ_SUPPLY_FLOW = "supply_flow"
 SZ_SUPPLY_TEMPERATURE = "supply_temperature"
 SZ_SPEED_CAP = "speed_cap"
 
+SZ_PRESENCE_DETECTED = "presence_detected"
+
 
 def slug(string: str) -> str:
     return re.sub(r"[\W_]+", "_", string.lower())
@@ -675,4 +677,7 @@ class Code(StrEnum):
     _3EF0 = "3EF0"
     _3EF1 = "3EF1"
     _4401 = "4401"
+    _4E01 = "4E01"
+    _4E02 = "4E02"
+    _4E04 = "4E04"
     _PUZZ = "7FFF"
