@@ -1327,6 +1327,19 @@ def parser_1fd4(payload, msg) -> dict:
     return {"ticker": int(payload[2:], 16)}
 
 
+@parser_decorator  # WIP: unknown, HVAC
+def parser_2210(payload, msg) -> dict:
+    # RP --- 32:153258 18:005904 --:------ 2210 042 00FF 00FFFFFF0000000000FFFFFFFFFF 00FFFFFF0000000000FFFFFFFFFF FFFFFF000000000000000800
+    # RP --- 32:153258 18:005904 --:------ 2210 042 00FF 00FFFF960000000003FFFFFFFFFF 00FFFF960000000003FFFFFFFFFF FFFFFF000000000000000800
+
+    assert payload in (
+        "00FF" + "00FFFFFF0000000000FFFFFFFFFF" * 2 + "FFFFFF000000000000000800",
+        "00FF" + "00FFFF960000000003FFFFFFFFFF" * 2 + "FFFFFF000000000000000800",
+    ), _INFORM_DEV_MSG
+
+    return {}
+
+
 @parser_decorator  # now_next_setpoint - Programmer/Hometronics
 def parser_2249(payload, msg) -> dict:
     # see: https://github.com/jrosser/honeymon/blob/master/decoder.cpp#L357-L370
@@ -1372,8 +1385,15 @@ def parser_22b0(payload, msg) -> dict:
 
 @parser_decorator  # ufh_setpoint, TODO: max length = 24?
 def parser_22c9(payload, msg) -> list:
+    # .I --- 02:001107 --:------ 02:001107 22C9 024 00-0834-0A28-01-0108340A2801-0208340A2801-0308340A2801  # noqa: E501
+    # .I --- 02:001107 --:------ 02:001107 22C9 006 04-0834-0A28-01
+
+    # .I --- 21:064743 --:------ 21:064743 22C9 006 00-07D0-0834-02
+    # .W --- 21:064743 02:250708 --:------ 22C9 006 03-07D0-0834-02
+    # .I --- 02:250708 21:064743 --:------ 22C9 008 03-07D0-7FFF-02-02-03
+
     def _parser(seqx) -> dict:
-        assert seqx[10:] == "01", f"is {seqx[10:]}, expecting 01"
+        assert seqx[10:] in ("01", "02"), f"is {seqx[10:]}, expecting 01/02"
 
         return {
             "temp_low": temp_from_hex(seqx[2:6]),
@@ -1390,15 +1410,27 @@ def parser_22c9(payload, msg) -> list:
             for i in range(0, len(payload), 12)
         ]
 
-    return _parser(payload)
+    return _parser(payload[:12])  # TODO: [12:]
 
 
 @parser_decorator  # unknown_22d0, HVAC system switch?
 def parser_22d0(payload, msg) -> dict:
 
-    assert payload == "00000002", _INFORM_DEV_MSG
+    # 2020-03-02T19:20:02.716392 056  I --- 02:001107 --:------ 02:001107 22D0 004 00000002          # an UFC
+
+    # 2022-07-28T13:25:40.196523 074  W --- 21:064743 02:250708 --:------ 22D0 008 0314001E-14030020
+    # 2022-07-28T13:25:40.237555 045  I --- 02:250708 21:064743 --:------ 22D0 004 03130000
+    # 2022-07-28T13:25:40.642563 045  I --- 02:250708 --:------ 02:250708 22D0 004 00130000          # sends 3x, 1s apart
+
+    assert payload in (
+        "00000002",
+        "00130000",
+        "03130000",
+        "0314001E14030020",
+    ), _INFORM_DEV_MSG
 
     return {
+        "idx": payload[:2],
         SZ_PAYLOAD: payload[2:],
     }
 
@@ -1408,6 +1440,43 @@ def parser_22d9(payload, msg) -> dict:
     return {SZ_SETPOINT: temp_from_hex(payload[2:6])}
 
 
+@parser_decorator  # WIP: unknown, HVAC
+def parser_22e0(payload, msg) -> dict:
+    # RP --- 32:155617 18:005904 --:------ 22E0 004 00-34-A0-1E
+    # RP --- 32:153258 18:005904 --:------ 22E0 004 00-64-A0-1E
+    def _parser(seqx) -> dict:
+        assert int(seqx, 16) <= 200 or seqx == "E6"  # only for 22E0, not 22E5/22E9
+        return int(seqx, 16) / 200
+
+    try:
+        return {
+            f"percent_{i}": percent_from_hex(payload[i : i + 2])
+            for i in range(2, len(payload), 2)
+        }
+    except ValueError:
+        return {
+            "percent_2": percent_from_hex(payload[2:4]),
+            "percent_4": _parser(payload[4:6]),
+            "percent_6": percent_from_hex(payload[6:8]),
+        }
+
+
+@parser_decorator  # WIP: unknown, HVAC
+def parser_22e5(payload, msg) -> dict:
+    # RP --- 32:153258 18:005904 --:------ 22E5 004 00-96-C8-14
+    # RP --- 32:155617 18:005904 --:------ 22E5 004 00-72-C8-14
+
+    return parser_22e0(payload, msg)
+
+
+@parser_decorator  # WIP: unknown, HVAC
+def parser_22e9(payload, msg) -> dict:
+    # RP --- 32:153258 18:005904 --:------ 22E9 004 00C8C814
+    # RP --- 32:155617 18:005904 --:------ 22E9 004 008CC814
+
+    return parser_22e0(payload, msg)
+
+
 @parser_decorator  # fan_speed (switch_mode), HVAC
 def parser_22f1(payload, msg) -> dict:
     # Orcon wireless remote 15RF
@@ -1429,9 +1498,7 @@ def parser_22f1(payload, msg) -> dict:
     # Scheme x: 0|x standby/off, 1|x min, 2+|x rate as % of max (Itho?)
     # Scheme 4: 0|4 standby/off, 1|4 auto, 2|4 low, 3|4 med, 4|4 high/boost
     # Scheme 7: only seen 000[2345]07 -- ? off, auto, rate x/4, +3 others?
-    # Scheme A: only seen 000[239A]0A -- ? off, auto, rate x/x, and?
-
-    # fan_mode = int(payload[2:4], 16)  # & 0b11110000
+    # Scheme A: only seen 000[239A]0A -- Normal, Boost (purge), HeaterOff & HeaterAuto
 
     try:
         assert payload[0:2] in ("00", "63")
@@ -1475,13 +1542,31 @@ def parser_22f1(payload, msg) -> dict:
 
     return {
         SZ_FAN_MODE: _22F1_FAN_MODE.get(payload[2:4], f"unknown_{payload[2:4]}"),
-        "_mode_idx": f"{int(payload[2:4], 16) & 0x07:02X}",
-        "_mode_max": payload[4:6] or None,
         "_scheme": _22f1_scheme,
+        "_mode_idx": f"{int(payload[2:4], 16) & 0x0F:02X}",
+        "_mode_max": payload[4:6] or None,
+        # "_payload": payload,
     }
 
 
-@parser_decorator  # switch_boost, HVAC
+@parser_decorator  # WIP: unknown, HVAC (flow rate?)
+def parser_22f2(payload, msg) -> dict:
+    # RP --- 32:155617 18:005904 --:------ 22F2 006 00-019B 01-0201
+    # RP --- 32:155617 18:005904 --:------ 22F2 006 00-0174 01-0208
+    # RP --- 32:155617 18:005904 --:------ 22F2 006 00-01E5 01-0201
+
+    def _parser(seqx) -> dict:
+        assert seqx[:2] in ("00", "01"), f"is {seqx[:2]}, expecting 00/01"
+
+        return {
+            "hvac_idx": seqx[:2],
+            "measure": temp_from_hex(seqx[2:]),
+        }
+
+    return [_parser(payload[i : i + 6]) for i in range(0, len(payload), 6)]
+
+
+@parser_decorator  # fan_boost, HVAC
 def parser_22f3(payload, msg) -> dict:
     # .I 019 --:------ --:------ 39:159057 22F3 003 00000A  # 10 mins
     # .I 022 --:------ --:------ 39:159057 22F3 003 000014  # 20 mins
@@ -1492,7 +1577,7 @@ def parser_22f3(payload, msg) -> dict:
 
     # NOTE: for boost timer for high
     try:
-        assert payload[2:4] in ("00", "02"), f"byte 1: {flag8(payload[2:4])}"
+        # assert payload[2:4] in ("00", "02", "12", "x52"), f"byte 1: {flag8(payload[2:4])}"
         assert msg.len <= 7 or payload[14:] == "0000", f"byte 7: {payload[14:]}"
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
@@ -1540,6 +1625,21 @@ def parser_22f3(payload, msg) -> dict:
     return result
 
 
+@parser_decorator  # WIP: unknown, HVAC
+def parser_22f4(payload, msg) -> dict:
+    # RP --- 32:155617 18:005904 --:------ 22F4 013 00-60E6-00000000000000-200000
+    # RP --- 32:153258 18:005904 --:------ 22F4 013 00-60DD-00000000000000-200000
+    # RP --- 32:155617 18:005904 --:------ 22F4 013 00-40B0-00000000000000-200000
+
+    assert payload[:2] == "00"
+    assert payload[6:] == "00000000000000200000"
+
+    return {
+        "value_02": payload[2:4],
+        "value_04": payload[4:6],
+    }
+
+
 @parser_decorator  # bypass_mode, HVAC
 def parser_22f7(payload, msg) -> dict:
     # RQ --- 37:171871 32:155617 --:------ 22F7 001 00
@@ -1658,8 +1758,19 @@ def parser_2401(payload, msg) -> dict:
 
 @parser_decorator  # unknown_2410, from OTB, FAN
 def parser_2410(payload, msg) -> dict:
-    # RP --- 10:048122 18:006402 --:------ 2410 020 000000-0000-00000000-00000001-00000001-00-000C  # OTB
-    # RP --- 32:155617 18:005904 --:------ 2410 020 000000-3EE8-00000000-FFFFFFFF-00000000-10-02A6  # Orcon Fan
+    # RP --- 10:048122 18:006402 --:------ 2410 020 00-00000000-00000000-00000001-00000001-00000C  # OTB
+    # RP --- 32:155617 18:005904 --:------ 2410 020 00-00003EE8-00000000-FFFFFFFF-00000000-1002A6  # Orcon Fan
+
+    def unstuff(seqx: str) -> tuple:
+        val = int(seqx, 16)
+        # if val & 0x40:
+        #     raise TypeError
+        signed = bool(val & 0x80)
+        length = (val >> 3 & 0x07) or 1
+        d_type = {0b000: "a", 0b001: "b", 0b010: "c", 0b100: "d"}.get(
+            val & 0x07, val & 0x07
+        )
+        return signed, length, d_type
 
     assert payload[:6] == "00" * 3, _INFORM_DEV_MSG
     assert payload[10:18] == "00" * 4, _INFORM_DEV_MSG
@@ -1667,7 +1778,14 @@ def parser_2410(payload, msg) -> dict:
     assert payload[26:34] in ("00000001", "00000000"), _INFORM_DEV_MSG
 
     return {
-        SZ_PAYLOAD: payload,
+        "tail": payload[34:],
+        "xxx_34": unstuff(payload[34:36]),
+        "xxx_36": unstuff(payload[36:38]),
+        "xxx_38": unstuff(payload[38:]),
+        "cur_value": payload[2:10],
+        "min_value": payload[10:18],
+        "max_value": payload[18:26],
+        "oth_value": payload[26:34],
     }
 
 
@@ -1809,11 +1927,13 @@ def parser_30c9(payload, msg) -> dict:
 
 @parser_decorator  # unknown_3110, HVAC
 def parser_3110(payload, msg) -> dict:
+    # .I --- 02:250708 --:------ 02:250708 3110 004 0000C820
+    # .I --- 21:042656 --:------ 21:042656 3110 004 00000020
 
     try:
         assert payload[2:4] == "00", f"byte 1: {payload[2:4]}"
         assert int(payload[4:6], 16) <= 200, f"byte 2: {payload[4:6]}"
-        assert payload[6:] in ("10", "20"), f"byte 3: {payload[6:]}"
+        assert payload[6:] in ("00", "10", "20"), f"byte 3: {payload[6:]}"
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
 
@@ -1848,6 +1968,22 @@ def parser_3120(payload, msg) -> dict:
     }
 
 
+@parser_decorator  # WIP: unknown, HVAC
+def parser_313e(payload, msg) -> dict:
+    # 11:00:59.412 RP --- 32:153258 18:005904 --:------ 313E 011 00-0000007937-003C80-0000
+    # 11:02:23.961 RP --- 32:153258 18:005904 --:------ 313E 011 00-0000007B14-003C80-0000
+    # 11:03:32.193 RP --- 32:153258 18:005904 --:------ 313E 011 00-0000007C1C-003C80-0000
+
+    assert payload[:2] == "00"
+    assert payload[12:] == "003C800000"
+
+    return {
+        "value_02": payload[2:12],
+        "value_12": payload[12:18],
+        "value_18": payload[18:],
+    }
+
+
 @parser_decorator  # datetime
 def parser_313f(payload, msg) -> dict:  # TODO: look for TZ
     # 2020-03-28T03:59:21.315178 045 RP --- 01:158182 04:136513 --:------ 313F 009 00FC3500A41C0307E4  # noqa: E501
@@ -1941,7 +2077,7 @@ def parser_31d9(payload, msg) -> dict:
 
     try:
         assert payload[8:32] in ("00" * 12, "20" * 12), f"byte 4: {payload[8:32]}"
-        assert payload[32:] in ("00", "08"), f"byte 16: {payload[32:]}"
+        assert payload[32:] in ("00", "04", "08"), f"byte 16: {payload[32:]}"
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
 
@@ -2066,21 +2202,31 @@ def parser_31e0(payload, msg) -> dict:
         break;
     }
     """
-    # .I --- 29:146052 32:023459 --:------ 31E0 003 00-00-00
-    # .I --- 29:146052 32:023459 --:------ 31E0 003 00-00-C8
 
-    # .I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
-    # .I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
-    # .I --- 32:166025 --:------ 30:079129 31E0 004 00-00-00-00
+    # .I --- 37:005302 32:132403 --:------ 31E0 008 00-0000-00 01-0064-00  # RF15 CO2 to Orcon HRC400 series SmartComfort Valve
 
-    # .I --- 32:168090 30:082155 --:------ 31E0 004 00-00-C8-00
-    # .I --- 37:258565 37:261128 --:------ 31E0 004 00-00-01-00
+    # .I --- 29:146052 32:023459 --:------ 31E0 003 00-0000
+    # .I --- 29:146052 32:023459 --:------ 31E0 003 00-00C8
 
-    return {
-        "vent_demand": percent_from_hex(payload[4:6]),
-        "flags_1": payload[2:4],
-        f"_{SZ_UNKNOWN}_3": payload[6:],
-    }
+    # .I --- 32:168240 30:079129 --:------ 31E0 004 00-0000-FF
+    # .I --- 32:168240 30:079129 --:------ 31E0 004 00-0000-FF
+    # .I --- 32:166025 --:------ 30:079129 31E0 004 00-0000-00
+
+    # .I --- 32:168090 30:082155 --:------ 31E0 004 00-00C8-00
+    # .I --- 37:258565 37:261128 --:------ 31E0 004 00-0001-00
+
+    def _parser(seqx) -> dict:
+        assert seqx[6:] in ("", "00", "FF")
+        return {
+            # "hvac_idx": seqx[:2],
+            "flags": seqx[2:4],
+            "vent_demand": percent_from_hex(seqx[4:6]),
+            f"_{SZ_UNKNOWN}_3": payload[6:],
+        }
+
+    if len(payload) > 8:
+        return [_parser(payload[x : x + 8]) for x in range(0, len(payload), 8)]
+    return _parser(payload)
 
 
 @parser_decorator  # supplied boiler water (flow) temp
@@ -2174,6 +2320,28 @@ def parser_3221(payload, msg) -> dict:
     }
 
 
+@parser_decorator  # WIP: unknown, HVAC
+def parser_3222(payload, msg) -> dict:
+    # 06:30:14.322 RP --- 32:155617 18:005904 --:------ 3222 004 00-00-01-00
+    # 00:09:26.263 RP --- 32:155617 18:005904 --:------ 3222 005 00-00-02-0009
+    # 02:42:27.090 RP --- 32:155617 18:005904 --:------ 3222 007 00-06-04-            000F100E
+    # 22:06:45.771 RP --- 32:155617 18:005904 --:------ 3222 011 00-02-08-    0009000F000F100E
+    # 13:30:26.792 RP --- 32:155617 18:005904 --:------ 3222 012 00-01-09-  090009000F000F100E
+    # 06:29:40.767 RP --- 32:155617 18:005904 --:------ 3222 013 00-00-0A-00090009000F000F100E
+
+    assert payload[:2] == "00"
+
+    if msg.len == 3:
+        assert payload[4:] == "00"
+        return {"percentage": percent_from_hex(payload[2:4])}
+
+    return {
+        "start": payload[2:4],
+        "length": payload[4:6],
+        "data": f"{'..' * int(payload[2:4])}{payload[6:]}",
+    }
+
+
 @parser_decorator  # unknown_3223, from OTB
 def parser_3223(payload, msg) -> dict:
 
@@ -2268,11 +2436,11 @@ def parser_3ef0(payload, msg) -> dict:
         # RP --- 10:138822 01:187666 --:------ 3EF0 006 000110FA00FF  # ?corrupt
 
         # for OTB (there's no reliable) modulation_level <-> flame_state)
-        assert (
-            payload[6:8] == "FF" or int(payload[6:8], 16) & 0b11110000 == 0
-        ), f"byte 3: {payload[6:8]}"
+        # assert (
+        #     payload[6:8] == "FF" or int(payload[6:8], 16) & 0b11110000 == 0
+        # ), f"byte 3: {payload[6:8]}"
         assert int(payload[8:10], 16) & 0b11110000 == 0, f"byte 4: {payload[8:10]}"
-        assert payload[10:12] in ("00", "1C", "FF"), f"byte 5: {payload[10:12]}"
+        # assert payload[10:12] in ("00", "1C", "FF"), f"byte 5: {payload[10:12]}"
 
         result.update(
             {
@@ -2365,7 +2533,10 @@ def parser_3ef1(payload, msg) -> dict:
 @parser_decorator  # timestamp, HVAC
 def parser_4401(payload, msg) -> dict:
 
-    assert payload[:4] == "1000", _INFORM_DEV_MSG
+    if msg.verb == RP:
+        return {}
+
+    # assert payload[:4] == "1000", _INFORM_DEV_MSG
     # assert payload[24:] == "0000000000000063", _INFORM_DEV_MSG
 
     return {
@@ -2376,6 +2547,21 @@ def parser_4401(payload, msg) -> dict:
     }  # epoch are in seconds
 
 
+@parser_decorator  # hvac_4e01
+def parser_4e01(payload, msg) -> dict:
+    return {f"val_{x}": temp_from_hex(payload[x : x + 4]) for x in range(2, 34, 4)}
+
+
+@parser_decorator  # hvac_4e02
+def parser_4e02(payload, msg) -> dict:
+
+    return (
+        {f"val_{x}": temp_from_hex(payload[x : x + 4]) for x in range(2, 34, 4)}
+        | {"val_34": payload[34:36]}
+        | {f"val_{x}": temp_from_hex(payload[x : x + 4]) for x in range(36, 68, 4)}
+    )
+
+
 # @parser_decorator  # faked puzzle pkt shouldn't be decorated
 def parser_7fff(payload, msg) -> dict:
 
