@@ -5,21 +5,26 @@
 
 Operates at the msg layer of: app - msg - pkt - h/w
 """
+from __future__ import annotations
 
 import asyncio
 import logging
+import signal
 from datetime import datetime as dt
 from datetime import timedelta as td
 from queue import Empty, Full, PriorityQueue, SimpleQueue
-from typing import Callable, List, Optional, Tuple
+from typing import Awaitable, Callable, Dict, Iterable, List, Optional, TypeVar
 
-from .command import ARGS, DEAMON, EXPIRES, FUNC, TIMEOUT, Command
-from .const import __dev_mode__
+from .command import Command
+from .const import SZ_DAEMON, SZ_EXPIRED, SZ_EXPIRES, SZ_FUNC, SZ_TIMEOUT, __dev_mode__
 from .exceptions import CorruptStateError, InvalidPacketError
 from .message import Message
+from .packet import Packet
 
 DONT_CREATE_MESSAGES = 3  # duplicate
 
+SZ_WRITER_TASK = "writer_task"
+
 DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
@@ -28,30 +33,69 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-class MakeCallbackAwaitable:
-    DEFAULT_TIMEOUT = 3  # in seconds
+_MessageProtocolT = TypeVar("_MessageProtocolT", bound="MessageProtocol")
+_MessageTransportT = TypeVar("_MessageTransportT", bound="MessageTransport")
 
-    def __init__(self, loop):
+
+class AwaitableCallback:
+    """Create an pair of functions so that the callback can be awaited.
+
+    The awaitable (getter) starts its `timeout` timer only when it is invoked.
+    It may raise a `TimeoutError` or a `TypeError`.
+
+    The callback (putter) may put the message in the queue before the getter is invoked.
+    """
+
+    SAFETY_TIMEOUT_DEFAULT = 30.0  # used to prevent waiting forever
+    SAFETY_TIMEOUT_MINIMUM = 10.0
+    HAS_TIMED_OUT = False
+    SHORT_WAIT = 0.001  # seconds
+
+    def __init__(self, loop) -> None:
         self._loop = loop or asyncio.get_event_loop()
-        self._queue = None
+        self._queue: SimpleQueue = SimpleQueue()  # unbounded, but we use only 1 entry
 
-    def create_pair(self) -> Tuple[Callable, Callable]:
-        self._queue = SimpleQueue()  # maxsize=1)
+        self.expires: dt = None  # type: ignore[assignment]
 
-        def putter(*args):  # callback
-            self._loop.call_soon_threadsafe(self._queue.put_nowait, args)
+    # the awaitable...
+    async def getter(self, timeout: float = SAFETY_TIMEOUT_DEFAULT) -> Message:
+        """Poll the queue until the message arrives, or the timer expires."""
 
-        async def getter(timeout=self.DEFAULT_TIMEOUT) -> Tuple:
-            timeout = self.DEFAULT_TIMEOUT if timeout is None else timeout
-            dt_expired = dt.now() + td(seconds=timeout)
-            while dt.now() < dt_expired:
-                try:
-                    return self._queue.get_nowait()
-                except Empty:
-                    await asyncio.sleep(0.005)
-            raise TimeoutError
+        if timeout is None or timeout <= self.SAFETY_TIMEOUT_MINIMUM:
+            timeout = self.SAFETY_TIMEOUT_DEFAULT
+        self.expires = dt.now() + td(seconds=timeout)
 
-        return getter, putter  # awaitable, callback
+        while dt.now() < self.expires:
+            try:
+                msg = self._queue.get_nowait()
+                break
+            except Empty:
+                await asyncio.sleep(self.SHORT_WAIT)
+        else:
+            raise TimeoutError(f"Safety timer expired (timeout={timeout}s)")
+
+        if msg is self.HAS_TIMED_OUT:
+            raise TimeoutError("Command timer expired")
+        if not isinstance(msg, Message):
+            raise TypeError(f"Response is not a message: {msg}")
+        return msg
+
+    # the callback...
+    def putter(self, msg: Message, timeout: float = SAFETY_TIMEOUT_DEFAULT) -> None:
+        """Put the message in the queue (when invoked)."""
+
+        if timeout is None or timeout <= self.SAFETY_TIMEOUT_MINIMUM:
+            timeout = self.SAFETY_TIMEOUT_DEFAULT
+        self.expires = dt.now() + td(seconds=timeout)
+
+        # self._queue.put_nowait(msg)  # ...so should not raise Full
+        self._loop.call_soon_threadsafe(self._queue.put_nowait, msg)
+
+
+def awaitable_callback(loop) -> tuple[Callable[..., Awaitable[Message]], Callable]:
+    """Create a pair of functions, so that a callback can be awaited."""
+    obj = AwaitableCallback(loop)
+    return obj.getter, obj.putter  # awaitable, callback
 
 
 class MessageTransport(asyncio.Transport):
@@ -73,51 +117,62 @@ class MessageTransport(asyncio.Transport):
 
     MAX_BUFFER_SIZE = 200
     MAX_SUBSCRIBERS = 3
-    WRITER_TASK = "writer_task"
 
-    def __init__(self, gwy, protocol, extra=None):
-        _LOGGER.debug("MsgTransport.__init__()")
+    READER = "receiver_callback"
+    WRITER = SZ_WRITER_TASK
+
+    _extra: dict  # asyncio.BaseTransport
+
+    def __init__(self, gwy, protocol: MessageProtocol, extra: dict = None) -> None:
+        super().__init__(extra=extra)
 
         self._loop = gwy._loop
 
         self._gwy = gwy
-        self._protocols = []
-        self.add_protocol(protocol)
+        self._protocols: List[MessageProtocol] = []
+        self._extra[self.READER] = self._pkt_receiver
+        self._dispatcher: Callable = None  # type: ignore[assignment]
 
-        self._extra = {} if extra is None else extra
-        self._is_closing = None
+        self._callbacks: Dict[str, dict] = {}
 
-        self._write_buffer_limit_high = None
-        self._write_buffer_limit_low = None
-        self._write_buffer_paused = None
-
-        self._callbacks = {}
-        self._dispatcher = None  # the HGI80 interface (is a asyncio.protocol)
-
-        self._que = PriorityQueue(maxsize=self.MAX_BUFFER_SIZE)
+        self._que: PriorityQueue = PriorityQueue(maxsize=self.MAX_BUFFER_SIZE)
+        self._write_buffer_limit_high: int = self.MAX_BUFFER_SIZE
+        self._write_buffer_limit_low: int = 0
+        self._write_buffer_paused = False
         self.set_write_buffer_limits()
 
-    def _set_dispatcher(self, dispatcher: Callable):
+        # self._extra[self.WRITER] = self._loop.create_task(self._polling_loop())
+
+        for sig in (signal.SIGINT, signal.SIGTERM):
+            self._loop.add_signal_handler(sig, self.abort)
+
+        self._is_closing = False
+
+        self.add_protocol(protocol)  # calls protocol.commection_made()
+
+    def _set_dispatcher(self, dispatcher: Callable) -> None:
         _LOGGER.debug("MsgTransport._set_dispatcher(%s)", dispatcher)
 
         async def call_send_data(cmd):
             _LOGGER.debug("MsgTransport.pkt_dispatcher(%s): send_data", cmd)
-            if cmd.callback:
-                self._add_callback(cmd.rx_header, cmd.callback)
+            if cmd._cbk:
+                self._add_callback(cmd.rx_header, cmd._cbk)
+
+            if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
+                _LOGGER.info("SENT: %s", cmd)
 
             await self._dispatcher(cmd)  # send_data, *once* callback registered
 
-            if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
-                _LOGGER.info("sent: %s", cmd)
-
         async def pkt_dispatcher():
+            """Poll the queue and send any command packets to the lower layer."""
             while True:
                 try:
                     cmd = self._que.get_nowait()
                 except Empty:
-                    if not self._is_closing:
-                        await asyncio.sleep(0.05)
-                        continue
+                    if self._is_closing:
+                        break
+                    await asyncio.sleep(0.05)
+                    continue
                 except AttributeError:  # when self._que == None, from abort()
                     break
 
@@ -136,117 +191,140 @@ class MessageTransport(asyncio.Transport):
             _LOGGER.error("MsgTransport.pkt_dispatcher(): connection_lost(None)")
             [p.connection_lost(None) for p in self._protocols]
 
-        self._dispatcher = dispatcher
-        self._extra[self.WRITER_TASK] = self._loop.create_task(pkt_dispatcher())
+        self._dispatcher = dispatcher  # type: ignore[assignment]
+        self._extra[self.WRITER] = self._loop.create_task(pkt_dispatcher())
 
-        return self._extra[self.WRITER_TASK]
+        return self._extra[self.WRITER]
 
-    def _add_callback(self, header, callback):
-        callback[EXPIRES] = (
+    def _add_callback(self, header: str, callback: dict) -> None:
+        callback[SZ_EXPIRES] = (
             dt.max
-            if callback.get(DEAMON)
-            else dt.now() + td(seconds=callback.get(TIMEOUT, 1))
+            if callback.get(SZ_DAEMON)
+            else dt.now() + td(seconds=callback.get(SZ_TIMEOUT, 1))
         )
         self._callbacks[header] = callback
 
-    def _pkt_receiver(self, pkt):
-        # _LOGGER.debug("MsgTransport._pkt_receiver(%s)", pkt)
+    def _pkt_receiver(self, pkt: Packet) -> None:
+        _LOGGER.debug("MsgTransport._pkt_receiver(%s)", pkt)
+
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
             _LOGGER.info("rcvd: %s", pkt)
 
+        # HACK: 1st, notify all expired callbacks
         for (
             hdr,
             callback,
-        ) in self._callbacks.items():  # 1st, notify all expired callbacks
-            if callback.get(EXPIRES, dt.max) < pkt.dtm and not callback.get("expired"):
+        ) in self._callbacks.items():
+            if callback.get(SZ_EXPIRES, dt.max) < pkt.dtm and not callback.get(
+                SZ_EXPIRED
+            ):
                 # see  also: PktProtocolQos.send_data()
-                _LOGGER.error("MsgTransport._pkt_receiver(%s): Expired callback", hdr)
-                callback[FUNC](False, *callback.get(ARGS, tuple()))
-                callback["expired"] = not callback.get(DEAMON, False)  # HACK:
+                _LOGGER.warning("MsgTransport._pkt_receiver(%s): Expired callback", hdr)
+                callback[SZ_FUNC](AwaitableCallback.HAS_TIMED_OUT)  # ZX: 1/3
+                callback[SZ_EXPIRED] = not callback.get(SZ_DAEMON, False)  # HACK:
 
-        self._callbacks = {  # 2nd, discard any expired callbacks
+        # HACK: 2nd, discard any expired callbacks
+        self._callbacks = {
             hdr: callback
             for hdr, callback in self._callbacks.items()
-            if callback.get(DEAMON)
-            or (callback[EXPIRES] >= pkt.dtm and not callback.get("expired"))
+            if callback.get(SZ_DAEMON)
+            or (callback[SZ_EXPIRES] >= pkt.dtm and not callback.get(SZ_EXPIRED))
         }
 
-        if len(self._protocols) == 0:
-            return
-
-        if self._gwy.config.reduce_processing >= DONT_CREATE_MESSAGES:
+        if len(self._protocols) == 0 or (
+            self._gwy.config.reduce_processing >= DONT_CREATE_MESSAGES
+        ):
             return
 
+        # BUG: all InvalidPacketErrors are not being raised here (see below)
         try:
             msg = Message(self._gwy, pkt)  # should log all invalid msgs appropriately
         except InvalidPacketError:
             return
 
+        # HACK: 3rd, invoke any callback
         # NOTE: msg._pkt._hdr is expensive - don't call it unless there's callbacks
         if self._callbacks and msg._pkt._hdr in self._callbacks:
-            callback = self._callbacks[msg._pkt._hdr]  # 3rd, invoke any callback
-            callback[FUNC](msg, *callback.get(ARGS, tuple()))
-            if not callback.get(DEAMON):
+            callback = self._callbacks[msg._pkt._hdr]
+            callback[SZ_FUNC](msg)  # ZX: 2/3
+            if not callback.get(SZ_DAEMON):
                 del self._callbacks[msg._pkt._hdr]
 
+        # BUG: the InvalidPacketErrors here should have been caught above
+        # BUG: should only need to catch CorruptStateError
         for p in self._protocols:
             try:
-                p.data_received(msg)
+                self._loop.call_soon(p.data_received, msg)
+
+            except InvalidPacketError:
+                return
+
             except CorruptStateError as exc:
                 _LOGGER.error("%s < %s", pkt, exc)
 
             except (  # protect this code from the upper-layer callback
-                ArithmeticError,  # incl. ZeroDivisionError,
                 AssertionError,
+            ) as exc:
+                if p is not self._protocols[0]:
+                    raise
+                _LOGGER.error("%s < exception from app layer: %s", pkt, exc)
+
+            except (  # protect this code from the upper-layer callback
+                ArithmeticError,  # incl. ZeroDivisionError,
                 AttributeError,
                 LookupError,  # incl. IndexError, KeyError
                 NameError,  # incl. UnboundLocalError
                 RuntimeError,  # incl. RecursionError
                 TypeError,
                 ValueError,
-            ) as exc:  # noqa: E722, broad-except
-                _LOGGER.exception("%s < exception from app layer: %s", pkt, exc)
+            ) as exc:
+                if p is self._protocols[0]:
+                    raise
+                _LOGGER.error("%s < exception from app layer: %s", pkt, exc)
 
-    def close(self):
+    def get_extra_info(self, name: str, default=None):
+        """Get optional transport information."""
+
+        return self._extra.get(name, default)
+
+    def abort(self) -> None:
+        """Close the transport immediately.
+
+        Buffered data will be lost. No more data will be received. The protocol's
+        connection_lost() method will (eventually) be called with None as its argument.
+        """
+
+        self._is_closing = True
+        self._clear_write_buffer()
+        self.close()
+
+    def close(self) -> None:
         """Close the transport.
 
         Buffered data will be flushed asynchronously. No more data will be received.
         After all buffered data is flushed, the protocol's connection_lost() method will
         (eventually) be called with None as its argument.
         """
-        _LOGGER.debug("MsgTransport.close()")
 
+        if self._is_closing:
+            return
         self._is_closing = True
 
-    def abort(self):
-        """Close the transport immediately.
+        self._pause_protocols()
+        if task := self._extra.get(self.WRITER):
+            task.cancel()
 
-        Buffered data will be lost. No more data will be received. The protocol's
-        connection_lost() method will (eventually) be called with None as its argument.
-        """
-        _LOGGER.debug("MsgTransport.abort(): clearing buffered data")
+        [self._loop.call_soon(p.connection_lost, None) for p in self._protocols]
 
-        self._is_closing = True
-        self._que = None
-
-    def is_closing(self) -> Optional[bool]:
+    def is_closing(self) -> bool:
         """Return True if the transport is closing or closed."""
-        _LOGGER.debug("MsgTransport.is_closing()")
-
         return self._is_closing
 
-    def get_extra_info(self, name, default=None):
-        """Get optional transport information."""
-        _LOGGER.debug("MsgTransport.get_extra_info(%s, %s)", name, default)
-
-        return self._extra.get(name, default)
-
-    def add_protocol(self, protocol):
-        """Set a new protocol.
+    def add_protocol(self, protocol: MessageProtocol) -> None:
+        """Attach a new protocol.
 
         Allow multiple protocols per transport.
         """
-        _LOGGER.debug("MsgTransport.add_protocol(%s)", protocol)
 
         if protocol not in self._protocols:
             if len(self._protocols) > self.MAX_SUBSCRIBERS - 1:
@@ -255,68 +333,67 @@ class MessageTransport(asyncio.Transport):
             self._protocols.append(protocol)
             protocol.connection_made(self)
 
-    def get_protocol(self) -> Optional[List]:
+    def get_protocols(self) -> list:
         """Return the list of active protocols.
 
         There can be multiple protocols per transport.
         """
-        _LOGGER.debug("MsgTransport.get_protocol()")
 
         return self._protocols
 
-    def is_reading(self) -> Optional[bool]:
+    def is_reading(self) -> bool:
         """Return True if the transport is receiving new data."""
-        _LOGGER.debug("MsgTransport.is_reading()")
 
         raise NotImplementedError
 
-    def pause_reading(self):
+    def pause_reading(self) -> None:
         """Pause the receiving end.
 
         No data will be passed to the protocol's data_received() method until
         resume_reading() is called.
         """
-        _LOGGER.debug("MsgTransport.pause_reading()")
 
         raise NotImplementedError
 
-    def resume_reading(self):
+    def resume_reading(self) -> None:
         """Resume the receiving end.
 
         Data received will once again be passed to the protocol's data_received()
         method.
         """
-        _LOGGER.debug("MsgTransport.resume_reading()")
 
         raise NotImplementedError
 
-    def _clear_write_buffer(self):
-        """Empty the dispatch queue."""
+    def _clear_write_buffer(self) -> None:
+        """Empty the dispatch queue.
+
+        Should not call `get_write_buffer_size()`.
+        """
 
-        self._pause_protocols()
         while not self._que.empty():
             try:
                 self._que.get_nowait()
             except Empty:
                 continue
             self._que.task_done()
-        self.get_write_buffer_size()
 
-    def _pause_protocols(self, force=None):
+    def _pause_protocols(self, force: bool = None) -> None:
         """Pause the other end."""
 
         if not self._write_buffer_paused or force:
             self._write_buffer_paused = True
-            [p.pause_writing() for p in self._protocols]
+            for p in self._protocols:
+                p.pause_writing()
 
-    def _resume_protocols(self, force=None):
+    def _resume_protocols(self, force: bool = None) -> None:
         """Resume the other end."""
 
         if self._write_buffer_paused or force:
             self._write_buffer_paused = False
-            [p.resume_writing() for p in self._protocols]
+            for p in self._protocols:
+                p.resume_writing()
 
-    def get_write_buffer_limits(self) -> Tuple[int, int]:
+    def get_write_buffer_limits(self) -> tuple[int, int]:
         """Get the high and low watermarks for write flow control.
 
         Return a tuple (low, high) where low and high are positive number of bytes.
@@ -324,7 +401,7 @@ class MessageTransport(asyncio.Transport):
 
         return self._write_buffer_limit_low, self._write_buffer_limit_high
 
-    def set_write_buffer_limits(self, high=None, low=None):
+    def set_write_buffer_limits(self, high: int = None, low: int = None) -> None:
         """Set the high- and low-water limits for write flow control.
 
         These two values control when to call the protocol's pause_writing() and
@@ -338,24 +415,20 @@ class MessageTransport(asyncio.Transport):
         empty. Use of zero for either limit is generally sub-optimal as it reduces
         opportunities for doing I/O and computation concurrently.
         """
-        _LOGGER.debug("MsgTransport.set_write_buffer_limits()")
 
-        self._write_buffer_limit_high = int(
-            self.MAX_BUFFER_SIZE
-            if high is None
-            else max((min((high, self.MAX_BUFFER_SIZE)), 0))
-        )
-        self._write_buffer_limit_low = int(
-            self._write_buffer_limit_high * 0.8
-            if low is None
-            else min((max((low, 0)), high))
-        )
+        high = self.MAX_BUFFER_SIZE if high is None else high
+        low = int(self._write_buffer_limit_high * 0.8) if low is None else low
+
+        self._write_buffer_limit_high = max((min((high, self.MAX_BUFFER_SIZE)), 0))
+        self._write_buffer_limit_low = min((max((low, 0)), high))
 
         self.get_write_buffer_size()
 
     def get_write_buffer_size(self) -> int:
-        """Return the current size of the write buffer."""
-        _LOGGER.debug("MsgTransport.get_write_buffer_size()")
+        """Return the current size of the write buffer.
+
+        If required, pause or resume the protocols.
+        """
 
         qsize = self._que.qsize()
 
@@ -367,7 +440,7 @@ class MessageTransport(asyncio.Transport):
 
         return qsize
 
-    def write(self, cmd):
+    def write(self, cmd: Command) -> None:
         """Write some data bytes to the transport.
 
         This does not block; it buffers the data and arranges for it to be sent out
@@ -379,14 +452,14 @@ class MessageTransport(asyncio.Transport):
             raise RuntimeError("MsgTransport is closing or has closed")
 
         if self._write_buffer_paused:
-            raise RuntimeError("MsgTransport: write buffer is paused")
+            raise RuntimeError("MsgTransport write buffer is paused")
 
         if self._gwy.config.disable_sending:
-            raise RuntimeError("MsgTransport: sending is disabled (cmd discarded)")
+            raise RuntimeError("MsgTransport sending is disabled (cmd discarded)")
 
         else:
-            if not self._dispatcher:  # TODO: do better?
-                _LOGGER.warning("MsgTransport.write(%s): no dispatcher", cmd)
+            # if not self._dispatcher:  # TODO: do better?
+            #     _LOGGER.debug("MsgTransport.write(%s): no dispatcher", cmd)
 
             try:
                 self._que.put_nowait(cmd)
@@ -395,30 +468,27 @@ class MessageTransport(asyncio.Transport):
 
         self.get_write_buffer_size()
 
-    def writelines(self, list_of_cmds):
+    def writelines(self, list_of_cmds: Iterable[Command]) -> None:
         """Write a list (or any iterable) of data bytes to the transport.
 
         The default implementation concatenates the arguments and calls write() on the
         result.list_of_cmds
         """
-        _LOGGER.debug("MsgTransport.writelines(%s)", list_of_cmds)
 
         for cmd in list_of_cmds:
             self.write(cmd)
 
-    def write_eof(self):
+    def write_eof(self) -> None:
         """Close the write end after flushing buffered data.
 
         This is like typing ^D into a UNIX program reading from stdin. Data may still be
         received.
         """
-        _LOGGER.debug("MsgTransport.write_eof()")
 
         raise NotImplementedError
 
     def can_write_eof(self) -> bool:
         """Return True if this transport supports write_eof(), False if not."""
-        _LOGGER.debug("MsgTransport.can_write_eof()")
 
         return False
 
@@ -449,78 +519,78 @@ class MessageProtocol(asyncio.Protocol):
     """
 
     def __init__(self, gwy, callback: Callable) -> None:
-        _LOGGER.debug("MsgProtocol.__init__(%s)", callback)
+
         # self._gwy = gwy  # is not used
         self._loop = gwy._loop
         self._callback = callback
 
-        self._transport = None
-        self._pause_writing = None
+        self._transport: MessageTransport = None  # type: ignore[assignment]
+        self._prev_msg: None | Message = None
+        self._this_msg: None | Message = None
 
-        self._prev_msg = None
-        self._this_msg = None
+        self._pause_writing = True
 
-    def connection_made(self, transport: MessageTransport) -> None:
+    def connection_made(self, transport: MessageTransport) -> None:  # type: ignore[override]
         """Called when a connection is made."""
-        _LOGGER.debug("MsgProtocol.connection_made(%s)", transport)
         self._transport = transport
+        self.resume_writing()
 
-    def data_received(self, msg: Message) -> None:
+    def data_received(self, msg: Message) -> None:  # type: ignore[override]
         """Called by the transport when a message is received."""
+        _LOGGER.debug("MsgProtocol.data_received(%s)", msg)
 
         self._this_msg, self._prev_msg = msg, self._this_msg
         self._callback(self._this_msg, prev_msg=self._prev_msg)
 
     async def send_data(
-        self, cmd: Command, awaitable=None, callback=None, **kwargs
+        self, cmd: Command, callback: Callable = None, _make_awaitable: bool = None
     ) -> Optional[Message]:
         """Called when a command is to be sent."""
         _LOGGER.debug("MsgProtocol.send_data(%s)", cmd)
 
-        if awaitable is not None and callback is not None:
+        if _make_awaitable and callback is not None:
             raise ValueError("only one of `awaitable` and `callback` can be provided")
 
-        if awaitable:
-            awaitable, callback = MakeCallbackAwaitable(self._loop).create_pair()
+        if _make_awaitable:  # and callback is None:
+            awaitable, callback = awaitable_callback(self._loop)  # ZX: 3/3
         if callback:  # func, args, daemon, timeout (& expired)
-            cmd.callback = {FUNC: callback, TIMEOUT: 3}
+            cmd._cbk = {SZ_FUNC: callback, SZ_TIMEOUT: 3}
 
         while self._pause_writing:
             await asyncio.sleep(0.005)
 
         self._transport.write(cmd)
 
-        if awaitable:
-            result = await awaitable(timeout=kwargs.get(TIMEOUT))  # may: TimeoutError
-            return result[0]  # a Message (or None/False?)
+        if _make_awaitable:
+            return await awaitable()  # AwaitableCallback.getter(timeout: float = ...)
+        return None
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         """Called when the connection is lost or closed."""
-        _LOGGER.debug("MsgProtocol.connection_lost(%s)", exc)
         if exc is not None:
             raise exc
 
     def pause_writing(self) -> None:
-        """Called by the transport when it's buffer goes over the high-water mark."""
-        _LOGGER.debug("MsgProtocol.pause_writing()")
+        """Called by the transport when its buffer goes over the high-water mark."""
         self._pause_writing = True
 
     def resume_writing(self) -> None:
-        """Called by the transport when it's buffer drains below the low-water mark."""
-        _LOGGER.debug("MsgProtocol.resume_writing()")
+        """Called by the transport when its buffer drains below the low-water mark."""
         self._pause_writing = False
 
 
-def create_protocol_factory(protocol: asyncio.Protocol, *args, **kwargs) -> Callable:
-    def _protocol_factory():
-        return protocol(*args, **kwargs)
+def create_protocol_factory(
+    protocol_class: type[asyncio.Protocol], *args, **kwargs
+) -> Callable:
+    def _protocol_factory() -> asyncio.Protocol:
+        return protocol_class(*args, **kwargs)
 
     return _protocol_factory
 
 
 def create_msg_stack(
-    gwy, msg_callback, protocol_factory=None
-) -> Tuple[asyncio.Protocol, asyncio.Transport]:
+    gwy, msg_callback: Callable, protocol_factory: Callable = None
+) -> tuple[_MessageProtocolT, _MessageTransportT]:
     """Utility function to provide a transport to a client protocol.
 
     The architecture is: app (client) -> msg -> pkt -> ser (HW interface).
