@@ -95,10 +95,11 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         # 19:20:49.476 045 RP --- 01:145038 12:010740 --:------ 000A 006 081001F40DAC
         EXPIRES: td(days=1),
     },
-    Code._000C: {  # zone_devices, TODO: needs I/RP
+    Code._000C: {  # zone_devices
         SZ_NAME: "zone_devices",
-        # RP --- 01:145038 18:013393 --:------ 000C 012 0100-00-10DAF5,0100-00-10DAFB
-        I_: r"^(0[0-9A-F][01][0-9A-F](0[0-9A-F]|7F)[0-9A-F]{6}){1,8}$",
+        # RP --- 01:145038 18:013393 --:------ 000C 018 06-08-00-1099C3 06-08-00-1099C5 06-08-00-1099BF
+        # RP --- 01:145038 18:013393 --:------ 000C 016 05-08-00-109901    08-00-109902    08-00-109903
+        I_: r"^0[0-9A-F][01][0-9A-F]|7F[0-9A-F]{6}([0-9A-F]{10}|[0-9A-F]{12}){1,7}$",
         RQ: r"^0[0-9A-F][01][0-9A-F]$",  # TODO: f"{zone_idx}{device_type}"
         EXPIRES: False,
     },
@@ -298,7 +299,7 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         # 2021-12-06T06:35:55.949502 071 RP --- 10:051349 18:135447 --:------ 3220 005 00C0130ECC
         SZ_NAME: "dhw_flow_rate",
         RQ: r"^00$",
-        RP: r"^00[0-9A-F](4)$",
+        RP: r"^00[0-9A-F]{4}$",
     },
     Code._1300: {  # cv water pressure (usu. for ch)
         SZ_NAME: "ch_pressure",
@@ -357,34 +358,65 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         SZ_NAME: "opentherm_sync",
         I_: r"^00([0-9A-F]{4})$",
     },
+    Code._2210: {  # unknown_2210, HVAC, NB: no I
+        SZ_NAME: "unknown_2210",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{82}$",
+    },
     Code._2249: {  # setpoint_now?
         SZ_NAME: "setpoint_now",  # setpt_now_next
         I_: r"^(0[0-9A-F]{13}){1,2}$",
     },  # TODO: This could be an array
     Code._22C9: {  # ufh_setpoint
-        # .I --- 02:001107 --:------ 02:001107 22C9 024 0008340A2801-0108340A2801-0208340A2801-0308340A2801  # noqa: E501
-        # .I --- 02:001107 --:------ 02:001107 22C9 006 04-0834-0A28-01
         SZ_NAME: "ufh_setpoint",
-        I_: r"^(0[0-9A-F][0-9A-F]{8}0[12]){1,4}$",  # ~000A array, but max_len 24, not 48!
+        I_: r"^(0[0-9A-F][0-9A-F]{8}0[12]){1,4}(0203)?$",  # ~000A array, but max_len 24, not 48!
+        W_: r"^(0[0-9A-F][0-9A-F]{8}0[12])$",  # ~000A array, but max_len 24, not 48!
         # RP: Appear wont get any?,
     },
     Code._22D0: {  # unknown_22d0, HVAC system switch?
         SZ_NAME: "message_22d0",
-        I_: r"^00",
+        I_: r"^(00|03)",
+        W_: r"^03",
     },
     Code._22D9: {  # boiler_setpoint
         SZ_NAME: "boiler_setpoint",
         RQ: r"^00$",
         RP: r"^00[0-9A-F]{4}$",
     },
+    Code._22E0: {  # unknown_22e0, HVAC, NB: no I
+        SZ_NAME: "unknown_22e0",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{6}$",
+    },
+    Code._22E5: {  # unknown_22e5, HVAC, NB: no I
+        SZ_NAME: "unknown_22e5",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{6}$",
+    },
+    Code._22E9: {  # unknown_22e9, HVAC, NB: no I
+        SZ_NAME: "unknown_22e9",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{6}$",
+    },
     Code._22F1: {  # fan_mode, HVAC
         SZ_NAME: "fan_mode",
+        RQ: r"^00$",
         I_: r"^(00|63)(0[0-9A-F]){1,2}$",
     },
+    Code._22F2: {  # unknown_22f2, HVAC, NB: no I
+        SZ_NAME: "unknown_22f2",
+        RQ: r"^00$",  # (00|01)?
+        RP: r"^00[0-9A-F]{4}(01[0-9A-F]{4})?$",
+    },
     Code._22F3: {  # fan_boost, HVAC
         SZ_NAME: "fan_boost",
         I_: r"^(00|63)[0-9A-F]{4}([0-9A-F]{8})?$",
     },  # minutes
+    Code._22F4: {  # unknown_22f4, HVAC, NB: no I
+        SZ_NAME: "unknown_22f4",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{24}$",
+    },
     Code._22F7: {  # fan_bypass_mode (% open), HVAC
         SZ_NAME: "fan_bypass_mode",
         I_: r"^00([0-9A-F]{2}){1,2}$",  # RP is the same
@@ -393,6 +425,7 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
     },
     Code._22F8: {  # fan_22f8 (moisture scenario?), HVAC
         SZ_NAME: "fan_22f8",
+        RQ: r"^00$",
         I_: r"^00[0-9A-F]{4}$",
     },
     Code._22B0: {  # programme_status, HVAC (1470, 1F70, 22B0)
@@ -486,6 +519,11 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         RQ: r"^00$",  # 20: will RP an RQ?
         # RP: r"^00[0-9A-F]{10}FF$",  # only ever: 20:/0070B000009CFF
     },
+    Code._313E: {  # unknown_313e, HVAC, NB: no I
+        SZ_NAME: "unknown_313e",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{20}$",
+    },
     Code._313F: {  # datetime (time report)
         SZ_NAME: "datetime",
         I_: r"^00[0-9A-F]{16}$",  # NOTE: RP is same
@@ -502,8 +540,8 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         SZ_NAME: "fan_state",
         # I_: r"^(00|21)[0-9A-F]{32}$",
         # I_: r"^(00|01|21)[0-9A-F]{4}((00|FE)(00|20){12}(00|08))?$",
-        I_: r"^(00|01|21)[0-9A-F]{4}(([0-9A-F]{2})(00|20){0,12}(00|08)?)?$",  # 00-0004-FE
-        RQ: r"^00$",
+        I_: r"^(00|01|21)[0-9A-F]{4}(([0-9A-F]{2})(00|20){0,12}(00|01|04|08)?)?$",  # 00-0004-FE
+        RQ: r"^(00|01|21)$",
     },
     Code._31DA: {  # hvac_state (fan_state_extended)
         SZ_NAME: "hvac_state",
@@ -516,7 +554,7 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         # 10:21:18.549 078  I --- 29:146052 32:023459 --:------ 31E0 003 000000
         # 07:56:50.522 095  I --- --:------ --:------ 07:044315 31E0 004 00006E00
         SZ_NAME: "fan_demand",
-        I_: r"^00[0-9A-F]{4}(00|FF)?$",
+        I_: r"^00([0-9A-F]{4}){1,3}(00|FF)?$",
     },
     Code._3200: {  # boiler output temp
         SZ_NAME: "boiler_output",
@@ -538,6 +576,11 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
         RQ: r"^00$",
         RP: r"^00",
     },
+    Code._3222: {  # unknown_3222, HVAC, NB: no I
+        SZ_NAME: "unknown_3222",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{4,20}$",
+    },
     Code._3223: {  # unknown_3223, from OTB
         SZ_NAME: "message_3223",
         RQ: r"^00$",
@@ -571,7 +614,22 @@ CODES_SCHEMA: dict[Code, dict] = {  # rf_unknown
     Code._4401: {  # unknown_4401 - HVAC
         SZ_NAME: "unknown_4401",
         I_: r"^[0-9A-F]{40}$",
-        RQ: r"^[0-9A-F]{40}$",  # NOTE: no RP!
+        RP: r"^00$",
+        RQ: r"^[0-9A-F]{40}$",
+        W_: r"^[0-9A-F]{40}$",
+    },
+    Code._4E01: {  # hvac_4e01 - HVAC
+        SZ_NAME: "hvac_4e01",
+        I_: r"^00([0-9A-F]{4}){8}00$",
+    },
+    Code._4E02: {  # hvac_4e02 - HVAC
+        SZ_NAME: "hvac_4e02",
+        I_: r"^00([0-9A-F]{4}){8}02([0-9A-F]{4}){8}$",
+    },
+    Code._4E04: {  # hvac_4e04 - HVAC
+        SZ_NAME: "hvac_4e04",
+        I_: r"^00(00FF|01FE)$",
+        W_: r"^00(00FF|01FE)$",
     },
     Code._PUZZ: {
         SZ_NAME: "puzzle_packet",
@@ -583,14 +641,14 @@ for code in CODES_SCHEMA.values():  # map any RPs to (missing) I_s
         code[RP] = code[I_]
 #
 # .I --- 01:210309 --:------ 01:210309 0009 006 FC00FFF900FF
-CODES_WITH_ARRAYS: dict[Code, list] = {
+CODES_WITH_ARRAYS: dict[Code, list] = {  # 000C/1FC9 are special
     Code._0005: [4, ("34",)],
     Code._0009: [3, ("01", "12", "22")],
     Code._000A: [6, ("01", "12", "22")],  # single element I after a W
     Code._2309: [3, ("01", "12", "22")],
     Code._30C9: [3, ("01", "12", "22")],
     Code._2249: [7, ("23",)],
-    Code._22C9: [6, ("02",)],  # *all* 22C9s are arrays (every 15min?)
+    Code._22C9: [6, ("02",)],
     Code._3150: [2, ("02",)],
 }  # TODO dex: element_length, src.type(s) (and dst.type too)
 #
@@ -636,7 +694,9 @@ CODE_IDX_SIMPLE: list[Code] = [
         or (I_ in v and v[I_].startswith(("^0[0-9A-F]", "^(0[0-9A-F]", "^((0[0-9A-F]")))
     )
 ]
-CODE_IDX_SIMPLE.extend((Code._10A0, Code._1260, Code._1F41, Code._3B00))
+CODE_IDX_SIMPLE.extend(
+    (Code._10A0, Code._1260, Code._1F41, Code._22D0, Code._31D9, Code._3B00)
+)
 # CODE_IDX_SIMPLE.sort()
 
 # IDX_NONE - *never has* a context: most payloads start 00, but no context even if the
@@ -962,6 +1022,7 @@ _DEV_KLASSES_HVAC: dict[str, dict] = {
     DEV_TYPE.FAN: {
         Code._0001: {RP: {}},
         Code._042F: {I_: {}},
+        Code._10D0: {I_: {}, RP: {}},
         Code._10E0: {I_: {}, RP: {}},
         Code._1298: {I_: {}},
         Code._12A0: {I_: {}},
@@ -1025,34 +1086,28 @@ CODES_BY_DEV_SLUG: dict[str, dict] = {
     **{k: v for k, v in _DEV_KLASSES_HEAT.items() if k is not None},
 }
 
-_CODES_EXCLUDED: tuple[Code, ...] = (
-    Code._0001,
-    Code._0002,
-    Code._000E,
-    Code._0016,
-    Code._0100,
-    Code._0150,
-    Code._01D0,
-    Code._01E9,
-    Code._0B04,
-    Code._10E0,
-    Code._1FC9,
+CODES_OF_HEAT_DOMAIN: tuple[Code] = sorted(  # type: ignore[assignment]
+    tuple(set(c for k in _DEV_KLASSES_HEAT.values() for c in k))
+    + (Code._0B04, Code._2389)
 )
-_CODES_HEAT: tuple[Code] = tuple(  # type: ignore[assignment]
-    dict.fromkeys(
-        c for k in _DEV_KLASSES_HEAT.values() for c in k if c not in _CODES_EXCLUDED
-    )
+CODES_OF_HVAC_DOMAIN: tuple[Code] = sorted(  # type: ignore[assignment]
+    tuple(set(c for k in _DEV_KLASSES_HVAC.values() for c in k))
+    + (Code._22F8, Code._4401, Code._4E01, Code._4E02, Code._4E04)
 )
-_CODES_HVAC: tuple[Code] = tuple(  # type: ignore[assignment]
-    dict.fromkeys(
-        c for k in _DEV_KLASSES_HVAC.values() for c in k if c not in _CODES_EXCLUDED
-    )
+CODES_OF_HEAT_DOMAIN_ONLY: tuple[Code, ...] = tuple(
+    c for c in sorted(CODES_OF_HEAT_DOMAIN) if c not in CODES_OF_HVAC_DOMAIN
 )
-CODES_HEAT_ONLY: tuple[Code, ...] = tuple(
-    c for c in _CODES_HEAT if c not in _CODES_HVAC
+CODES_OF_HVAC_DOMAIN_ONLY: tuple[Code, ...] = tuple(
+    c for c in sorted(CODES_OF_HVAC_DOMAIN) if c not in CODES_OF_HEAT_DOMAIN
 )
-CODES_HVAC_ONLY: tuple[Code, ...] = tuple(
-    c for c in _CODES_HVAC if c not in _CODES_HEAT
+_CODES_OF_BOTH_DOMAINS: tuple[Code, ...] = sorted(
+    tuple(set(CODES_OF_HEAT_DOMAIN) & set(CODES_OF_HVAC_DOMAIN))
+)
+_CODES_OF_EITHER_DOMAIN: tuple[Code, ...] = sorted(
+    tuple(set(CODES_OF_HEAT_DOMAIN) | set(CODES_OF_HVAC_DOMAIN))
+)
+_CODES_OF_NO_DOMAIN: tuple[Code, ...] = tuple(
+    c for c in CODES_SCHEMA if c not in _CODES_OF_EITHER_DOMAIN
 )
 
 _CODE_FROM_NON_CTL: tuple[Code] = tuple(  # type: ignore[assignment]
@@ -1120,14 +1175,11 @@ _22F1_MODE_ITHO: dict[str, str] = {
 }
 
 _22F1_MODE_NUAIRE: dict[str, str] = {
-    "00": "standby",
-    "01": "auto",
-    "02": "low",
-    "03": "medium",
-    "04": "high",  # boost
+    "02": "normal",
+    "03": "boost",  # aka purge
     "09": "heater_off",
     "0A": "heater_auto",
-}
+}  # DRI-ECO-2S (normal/boost only), DRI-ECO-4S
 
 _22F1_MODE_ORCON: dict[str, str] = {
     "00": "away",
