@@ -34,7 +34,7 @@ from queue import Queue
 from string import printable  # ascii_letters, digits
 from time import perf_counter
 from types import SimpleNamespace
-from typing import Awaitable, Callable, Iterable, Optional, TypeVar
+from typing import Awaitable, Callable, Iterable, Optional, TextIO, TypeVar
 
 from serial import SerialBase, SerialException, serial_for_url  # type: ignore[import]
 from serial_asyncio import SerialTransport as SerTransportAsync  # type: ignore[import]
@@ -43,14 +43,15 @@ from .address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR
 from .command import Command, Qos
 
 # skipcq: PY-W2000
-from .const import DEV_TYPE_MAP, SZ_DEVICE_ID, __dev_mode__
+from .const import DEV_TYPE, DEV_TYPE_MAP, SZ_DEVICE_ID, __dev_mode__
 from .exceptions import InvalidPacketError
 from .helpers import dt_now
 from .packet import Packet
 from .protocol import create_protocol_factory
 from .schemas import (
-    SCH_CONFIG_SERIAL,
+    SCH_SERIAL_PORT_CONFIG,
     SZ_BLOCK_LIST,
+    SZ_CLASS,
     SZ_INBOUND,
     SZ_KNOWN_LIST,
     SZ_OUTBOUND,
@@ -89,8 +90,9 @@ _DEFAULT_USE_REGEX = {
 
 TIP = f", configure the {SZ_KNOWN_LIST}/{SZ_BLOCK_LIST} as required"
 
-IS_INITIALIZED = "is_initialized"
-IS_EVOFW3 = "is_evofw3"
+SZ_FINGERPRINT = "fingerprint"
+SZ_IS_EVOFW3 = "is_evofw3"
+SZ_KNOWN_HGI = "known_hgi"
 
 ERR_MSG_REGEX = re.compile(r"^([0-9A-F]{2}\.)+$")
 
@@ -484,20 +486,22 @@ class PacketProtocolBase(asyncio.Protocol):
         self._evofw_flag = getattr(gwy.config, "evofw_flag", None)
 
         self.enforce_include = gwy.config.enforce_known_list
-        if self.enforce_include:
-            self._exclude = list(gwy._exclude.keys())
-            self._include = list(gwy._include.keys())
-        else:
-            self._exclude = list(gwy._exclude.keys())
-            self._include = []
+        self._exclude = list(gwy._exclude.keys())
+        self._include = list(gwy._include.keys()) + [NON_DEV_ADDR.id, NUL_DEV_ADDR.id]
         self._unwanted: list = []  # not: [NON_DEV_ADDR.id, NUL_DEV_ADDR.id]
 
         self._hgi80 = {
-            IS_INITIALIZED: False,
-            IS_EVOFW3: None,
-            SZ_DEVICE_ID: HGI_DEV_ADDR.id,
+            SZ_DEVICE_ID: None,
+            SZ_FINGERPRINT: None,
+            SZ_IS_EVOFW3: None,
+            SZ_KNOWN_HGI: None,
         }  # also: "evofw3_ver"
 
+        if known_hgis := [
+            k for k, v in gwy._include.items() if v.get(SZ_CLASS) == DEV_TYPE.HGI
+        ]:
+            self._hgi80[SZ_KNOWN_HGI] = known_hgis[0]
+
         self._use_regex = getattr(self._gwy.config, SZ_USE_REGEX, {})
 
     def __repr__(self) -> str:
@@ -516,7 +520,9 @@ class PacketProtocolBase(asyncio.Protocol):
 
         self._transport = transport
 
-        if self._include:  # TODO: here, or in init?
+        # self.resume_writing()  # executed in selected sub-classes
+
+        if self.enforce_include:  # TODO: here, or in init?
             _LOGGER.info(
                 f"Enforcing the {SZ_KNOWN_LIST} (as a whitelist): %s", self._include
             )
@@ -535,7 +541,7 @@ class PacketProtocolBase(asyncio.Protocol):
         _LOGGER.debug(f"{self}.connection_lost(exc)")
         # serial.serialutil.SerialException: device reports error (poll)
 
-        self._pause_writing = True
+        self.pause_writing()
 
         if exc is not None:
             raise exc
@@ -566,28 +572,44 @@ class PacketProtocolBase(asyncio.Protocol):
         for dtm, raw_line in bytes_received(data):
             self._line_received(dtm, _normalise(_str(raw_line)), raw_line)
 
-    def _check_set_hgi80(self, pkt):
-        """Check/set HGI; log if it is a foreign HGI."""
-        assert pkt.src.type == DEV_TYPE_MAP.HGI and pkt.src.id != HGI_DEV_ADDR.id
+    def _check_set_hgi80(self, pkt: Packet):
+        """Check/set (a pkt with) src.type == 18: - is it the HGI, or a foreign HGI?"""
 
-        if self._hgi80[SZ_DEVICE_ID] is HGI_DEV_ADDR.id:
-            self._hgi80[SZ_DEVICE_ID] = pkt.src.id
+        if pkt.src.id == self._hgi80[SZ_DEVICE_ID]:
+            return
 
-        elif (
-            self._hgi80[SZ_DEVICE_ID] != pkt.src.id and pkt.src.id not in self._unwanted
-        ):
-            _LOGGER.debug(
-                f"{pkt} < There appears to be more than one HGI80-compatible device"
-                f" (active gateway: {self._hgi80[SZ_DEVICE_ID]}), this is unsupported{TIP}"
+        if self._hgi80[SZ_DEVICE_ID]:
+            if pkt.verb != RQ:  # HACK: to reduce logspam
+                return
+
+            _LOGGER.warning(  # more than one GWY
+                f"{pkt} < Appears to be a Foreign gateway: {pkt.src.id} (Active gateway: "
+                f"{self._hgi80[SZ_DEVICE_ID]}): this is unsupported{TIP}"
             )
 
+            if pkt.src.id not in self._include and self.enforce_include:
+                _LOGGER.info(f"Blacklisting {pkt.src.id} (is Foreign gateway?){TIP}")
+                self._unwanted.append(pkt.src.id)
+            return
+
+        if pkt.payload == self._hgi80[SZ_FINGERPRINT]:
+            self._hgi80[SZ_DEVICE_ID] = pkt.src.id
+            _LOGGER.warning(f"{pkt} < Active gateway set to: {pkt.src.id}")
+
+            if pkt.src.id not in self._include:
+                _LOGGER.info(f"Whitelisting {pkt.src.id} (is Active gateway){TIP}")
+                self._include.append(pkt.src.id)  # NOTE: only time _include is modified
+            return
+
+        if pkt.src.id == self._hgi80[SZ_KNOWN_HGI]:
+            self._hgi80[SZ_DEVICE_ID] = pkt.src.id
+            _LOGGER.warning(f"{pkt} < Active gateway set to: {pkt.src.id}")
+
     def _line_received(self, dtm: dt, line: str, raw_line: bytes) -> None:
 
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
             _LOGGER.info("RF Rx: %s", raw_line)
 
-        self._hgi80[IS_INITIALIZED], was_initialized = True, self._hgi80[IS_INITIALIZED]
-
         try:
             pkt = Packet.from_port(
                 self._gwy,
@@ -596,18 +618,15 @@ class PacketProtocolBase(asyncio.Protocol):
                 raw_line=raw_line,
             )  # should log all? invalid pkts appropriately
 
-            if pkt.src.type == DEV_TYPE_MAP.HGI:
-                self._check_set_hgi80(pkt)
         except InvalidPacketError as exc:
-            if "# evofw" in line and self._hgi80[IS_EVOFW3] is None:
-                self._hgi80[IS_EVOFW3] = True
+            if "# evofw" in line and self._hgi80[SZ_IS_EVOFW3] is None:
+                self._hgi80[SZ_IS_EVOFW3] = True
                 self._hgi80["evofw3_ver"] = line
                 if self._evofw_flag not in (None, "!V"):
                     self._transport.write(
                         bytes(f"{self._evofw_flag}\r\n".encode("ascii"))
                     )
-            elif was_initialized and line and line[:1] != "#" and "*" not in line:
-                _LOGGER.error("%s < Cant create packet (ignoring): %s", line, exc)
+            _LOGGER.debug("%s < Cant create packet (ignoring): %s", line, exc)
             return
 
         self._pkt_received(pkt)
@@ -618,19 +637,14 @@ class PacketProtocolBase(asyncio.Protocol):
         self._this_pkt, self._prev_pkt = pkt, self._this_pkt
 
         if self._callback and self._is_wanted(pkt.src.id, pkt.dst.id):
-            try:
+
+            if pkt.src.type == DEV_TYPE_MAP.HGI:
+                self._check_set_hgi80(pkt)
+
+            try:  # why not call soon?
                 self._callback(pkt)  # only wanted PKTs to the MSG transport's handler
 
-            except (  # protect this code from the upper-layer callback
-                # ArithmeticError,  # incl. ZeroDivisionError,
-                AssertionError,
-                # AttributeError,
-                # LookupError,  # incl. IndexError, KeyError
-                # # NameError,  # incl. UnboundLocalError
-                # RuntimeError,  # incl. RecursionError
-                # TypeError,
-                # ValueError,
-            ) as exc:  # noqa: E722, broad-except
+            except AssertionError as exc:  # protect from upper-layer callbacks
                 _LOGGER.exception("%s < exception from msg layer: %s", pkt, exc)
 
     def _is_wanted(self, src_id: str, dst_id: str) -> bool:
@@ -642,41 +656,19 @@ class PacketProtocolBase(asyncio.Protocol):
 
         for dev_id in dict.fromkeys((src_id, dst_id)):
             if dev_id in self._unwanted:  # TODO: remove entries older than (say) 1w/1d
-                _LOGGER.debug(f"Blocked {dev_id} (is unwanted){TIP}")
                 return False
 
             if dev_id in self._exclude:
-                _LOGGER.info(f"Blocking {dev_id} (in {SZ_BLOCK_LIST})")
                 self._unwanted.append(dev_id)
                 return False
 
-            if dev_id in self._include or dev_id in (NON_DEV_ADDR.id, NUL_DEV_ADDR.id):
-                # _LOGGER.debug(f"Allowed {dev_id} (in {SZ_BLOCK_LIST}, or is the gateway")
+            if dev_id in self._include:
+                continue  # TODO: or break if not self.enforce_include?
+
+            if not self._hgi80[SZ_KNOWN_HGI] and dev_id[:2] == DEV_TYPE_MAP.HGI:
                 continue
 
-            if dev_id == self._hgi80[SZ_DEVICE_ID]:
-                if self._include:
-                    _LOGGER.warning(f"Allowing {dev_id} (is the gateway){TIP}")
-                self._include.append(dev_id)  # NOTE: only time include list is modified
-                continue
-
-            if dev_id[:2] == DEV_TYPE_MAP.HGI and not self._hgi80[SZ_DEVICE_ID]:
-                _LOGGER.debug(f"Allowed {dev_id} (is a gateway?){TIP}")
-                continue
-
-            if dev_id[:2] == DEV_TYPE_MAP.HGI and self._gwy.ser_name:  # dex
-                (_LOGGER.debug if dev_id in self._exclude else _LOGGER.warning)(
-                    f"Blocking {dev_id} (is a foreign gateway){TIP}"
-                )
-                self._unwanted.append(dev_id)
-                return False
-
-            if self.enforce_include:
-                if self._exclude:
-                    _LOGGER.debug(f"Blocking {dev_id} (not in {SZ_BLOCK_LIST}){TIP}")
-                else:
-                    _LOGGER.debug(f"Blocking {dev_id} (no {SZ_BLOCK_LIST}){TIP}")
-                self._unwanted.append(dev_id)
+            if self.enforce_include:  # TODO: omit if using break?
                 return False
 
         return True
@@ -725,10 +717,6 @@ class PacketProtocolFile(PacketProtocolBase):
         except (InvalidPacketError, ValueError):  # VE from dt.fromisoformat()
             return
 
-        if (
-            pkt.src.type == DEV_TYPE_MAP.HGI and pkt.src.id != HGI_DEV_ADDR.id
-        ):  # HACK 01: dex
-            self._check_set_hgi80(pkt)
         self._pkt_received(pkt)
 
 
@@ -767,12 +755,13 @@ class PacketProtocolPort(PacketProtocolBase):
         super().connection_made(transport)  # self._transport = transport
         # self._transport.serial.rts = False
 
-        # determine if using a evofw3 rather than a HGI80
-        self._transport.write(bytes("!V\r\n".encode("ascii")))
+        self._transport.write(bytes("!V\r\n".encode("ascii")))  # is evofw3 or HGI80?
 
         # add this to start of the pkt log, if any
-        if False and not self._disable_sending:
-            self._transport.write(bytes(str(Command._puzzle()), "ascii") + b"\r\n")
+        if not self._disable_sending:
+            cmd = Command._puzzle()
+            self._hgi80[SZ_FINGERPRINT] = cmd.payload
+            self._transport.write(bytes(str(cmd), "ascii") + b"\r\n")
 
         self.resume_writing()
 
@@ -793,7 +782,7 @@ class PacketProtocolPort(PacketProtocolBase):
 
     async def _handle_impersonation(self, cmd: Command) -> None:
         msg = f"Impersonating device: {cmd.src}, for pkt: {cmd.tx_header}"
-        if self._hgi80[IS_EVOFW3]:
+        if self._hgi80[SZ_IS_EVOFW3]:
             _LOGGER.info(msg)
         else:
             _LOGGER.warning(
@@ -826,7 +815,7 @@ class PacketProtocolPort(PacketProtocolBase):
         await self._sem.acquire()  # minimum time between Tx
 
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
-            _LOGGER.info("RF Tx:     %s", data)
+            _LOGGER.info("RF Tx:     %s", data_bytes)
         self._transport.write(data_bytes + b"\r\n")
 
 
@@ -932,8 +921,9 @@ def create_pkt_stack(
     /,
     *,
     protocol_factory: Callable = None,
-    ser_port: str = None,
-    packet_log=None,
+    port_name: str = None,
+    port_config: dict = None,
+    packet_log: TextIO = None,
     packet_dict: dict = None,
 ) -> tuple[_PacketProtocolT, _PacketTransportT]:
     """Utility function to provide a transport to the internal protocol.
@@ -951,7 +941,7 @@ def create_pkt_stack(
         # - python client.py monitor 'rfc2217://localhost:5001'
         # - python client.py monitor 'alt:///dev/ttyUSB0?class=PosixPollSerial'
 
-        ser_config = SCH_CONFIG_SERIAL(ser_config)
+        ser_config = SCH_SERIAL_PORT_CONFIG(ser_config or {})
 
         try:
             ser_obj = serial_for_url(ser_name, **ser_config)
@@ -991,15 +981,15 @@ def create_pkt_stack(
                 PacketProtocolQos, gwy, pkt_callback
             )()  # NOTE: should be: PacketProtocolQos, not PacketProtocolPort
 
-    if len([x for x in (packet_dict, packet_log, ser_port) if x is not None]) != 1:
-        raise TypeError("serial port, log file & dict should be mutually exclusive")
+    if len([x for x in (packet_dict, packet_log, port_name) if x is not None]) != 1:
+        raise TypeError("must have exactly one of: serial port, pkt log or pkt dict")
 
     pkt_protocol = (protocol_factory or protocol_factory_)()
 
     if (pkt_source := packet_log or packet_dict) is not None:  # {} is a processable log
         return pkt_protocol, SerTransportRead(gwy._loop, pkt_protocol, pkt_source)
 
-    ser_instance = get_serial_instance(ser_port, gwy._port_config)
+    ser_instance = get_serial_instance(port_name, port_config)
 
     if os.name == "nt" or ser_instance.portstr[:7] in ("rfc2217", "socket:"):
         issue_warning()
